@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2007 the xine project
+ * Copyright (C) 2000-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -25,6 +25,7 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <string.h>
+#include <strings.h>
 #include <stdlib.h>
 #include <dlfcn.h>
 #include <errno.h>
@@ -359,13 +360,14 @@ NPError NPP_New(NPMIMEType pluginType, NPP instance,
   this = (plugin_instance_t*) instance->pdata;
 
   globals.url           = NULL;
-  this->real_controls = NULL;
-  this->autostart     = 0;
 
   if (this != NULL) {
 
     int i;
 
+    this->real_controls = NULL;
+    this->autostart     = 0;
+
     /* parse args */
 
     this->emu_mode = EMU_NONE;
@@ -478,7 +480,7 @@ static void url_cb (Widget w, XtPointer client_data, XtPointer call_data) {
 NPError NPP_SetWindow(NPP instance, NPWindow* window) {
 
   plugin_instance_t* this;
-  Widget hello, button, box;
+  Widget /*hello,*/ button, box;
 
   xprintf("NPP_SetWindow: 42\n");
 
@@ -502,7 +504,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window) {
   this->screen    = XtScreen (this->top_level);
 
   xprintf("x=%u, y=%u, w=%u, h=%u\n", window->x, window->y, window->width, window->height);
-  xprintf("window = %u NPERR_NO_ERROR\n", this->window);
+  xprintf("window = %lu NPERR_NO_ERROR\n", this->window);
 
   this->black = BlackPixelOfScreen (this->screen);
   this->white = WhitePixelOfScreen (this->screen);
@@ -518,7 +520,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window) {
 				 XtNwidth, this->width,
 				 XtNheight, this->height, NULL);
 
-  hello = XtVaCreateManagedWidget ("gxine browser plugin", labelWidgetClass, box,
+  /*hello = */XtVaCreateManagedWidget ("gxine browser plugin", labelWidgetClass, box,
 				   XtNbackground, this->black,
 				   XtNforeground, this->white,
 				   XtNwidth, this->width,
@@ -655,11 +657,12 @@ int32 STREAMBUFSIZE = 0X0FFFFFFF; /* If we are reading from a file in NPAsFile
 
 int32 NPP_WriteReady(NPP instance, NPStream *stream) {
 
-  plugin_instance_t* this;
+  /*plugin_instance_t* this;*/
   xprintf("NPP_WriteReady:\n");
+  /*
   if (instance != NULL)
     this = (plugin_instance_t*) instance->pdata;
-
+  */
   return STREAMBUFSIZE;
 }
 
@@ -679,24 +682,27 @@ int32 NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *b
 
 NPError NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason) {
 
-  plugin_instance_t* this;
+  /*plugin_instance_t* this; */
 
   xprintf("NPP_DestroyStream: \n");
   if (instance == NULL)
     return NPERR_INVALID_INSTANCE_ERROR;
+  /*
   this = (plugin_instance_t*) instance->pdata;
-
+  */
   return NPERR_NO_ERROR;
 }
 
 
 void NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname) {
 
-  plugin_instance_t* this;
+  /*plugin_instance_t* this;*/
 
   xprintf("NPP_StreamAsFile:\n");
+  /*
   if (instance != NULL)
     this = (plugin_instance_t*) instance->pdata;
+  */
 }
 
 
