@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2007 the xine-project
+ * Copyright (C) 2003-2017 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,7 +38,7 @@
 
 se_t *gse; /* global script engine */
 xine_t *xine; /* global xine instance */
-GStaticRecMutex engine_lock = G_STATIC_REC_MUTEX_INIT;
+GRecMutex engine_lock;
 
 gboolean initialised = FALSE;
 
@@ -61,6 +61,7 @@ show_about (JSContext *cx, uintN argc, jsval *vp)
   static const gchar *const authors[] = {
     "Darren Salt <dsalt@users.sourceforge.net>",
     "Guenter Bartsch <guenter@users.sourceforge.net>",
+    "Petri Hintukainen <phintuka@users.sourceforge.net>",
     NULL
   };
   static const gchar *const documenters[] = {
@@ -110,14 +111,14 @@ show_about (JSContext *cx, uintN argc, jsval *vp)
 		  strcmp (translator, "TRANSLATOR") ? translator : NULL,
 		  "logo-icon-name", GXINE_LOGO, NULL);
 
-    char *version_label =
+    gchar *version_label =
       g_strdup_printf (_("<small><i>%s\nUsing xine-lib %s, built with %s</i></small>"),
 		       VENDOR_PKG_VERSION CSET_ID,
 		       xine_get_version_string (), XINE_VERSION);
     label = ui_label_new_with_markup (version_label);
     gtk_label_set_justify ((GtkLabel *) label, GTK_JUSTIFY_CENTER);
-    free (version_label);
-    vbox = GTK_BOX (GTK_DIALOG (about)->vbox);
+    g_free (version_label);
+    vbox = GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG (about)));
     gtk_box_pack_end (vbox, label, FALSE, FALSE, 2);
     g_object_connect (about,
 	"signal::delete-event", G_CALLBACK (gtk_widget_hide_on_delete), NULL,
@@ -161,7 +162,7 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
     g_warning ("JS execution during startup: %s: %s", src, cmd);
 #endif
 
-  if (!g_static_rec_mutex_trylock (&engine_lock))
+  if (!g_rec_mutex_trylock (&engine_lock))
   {
     g_critical (_("JS execution attempted while lock already held.\n"
 		  "Try to reproduce this in a debugger (set a breakpoint on g_log).\n"
@@ -188,7 +189,7 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
     str = g_strdup_printf (_("result: %s"), str);
     cb (cb_data, "%s", str);
     free (free_str);
-    free (str);
+    g_free (str);
   }
   else if (se_result_double (gse, &num.d))
   {
@@ -216,11 +217,11 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
   else
   {
     /* return 0 iff the result type wasn't recognised */
-    g_static_rec_mutex_unlock (&engine_lock);
+    g_rec_mutex_unlock (&engine_lock);
     return 0;
   }
 
-  g_static_rec_mutex_unlock (&engine_lock);
+  g_rec_mutex_unlock (&engine_lock);
   return 1;
 }
 
@@ -269,33 +270,15 @@ typedef struct {
   /*const*/ char *src;
 } exec_t;
 
-void engine_queue_push (const char *command, se_o_t *obj, se_print_cb_t cb,
-			void *cb_data, se_error_cb_t ecb, const char *src)
-{
-  exec_t *data = malloc (sizeof (exec_t));
-  *data = (exec_t){ g_strdup (command), obj, cb, cb_data, ecb, g_strdup (src) };
-  g_async_queue_push (js_queue, data);
-
-  GdkEvent event;
-  event.client.type = GDK_CLIENT_EVENT;
-  event.client.window = app->window;
-  event.client.send_event = TRUE;
-  event.client.message_type = GDK_NONE;
-  event.client.data_format = 8;
-  strcpy (event.client.data.b, "Javascript");
-  gdk_event_put (&event);
-}
-
-static gboolean js_queue_cb (GtkWidget *widget, GdkEventClient *event,
-			    gpointer data)
+static void js_queue_cb (void)
 {
   /* js_lock must be recursive since, if the engine is busy, this function
    * WILL be re-entered from the same thread with the lock already held.
    */
-  static GStaticRecMutex js_lock = G_STATIC_REC_MUTEX_INIT;
+  static GRecMutex js_lock;
   exec_t *js;
 
-  g_static_rec_mutex_lock (&js_lock);
+  g_rec_mutex_lock (&js_lock);
   while ((js = g_async_queue_try_pop (js_queue)))
   {
     /* This function is entered with the GDK lock held. This is bad, from
@@ -314,9 +297,46 @@ static gboolean js_queue_cb (GtkWidget *widget, GdkEventClient *event,
     free (js->src);
     free (js);
   }
-  g_static_rec_mutex_unlock (&js_lock);
+  g_rec_mutex_unlock (&js_lock);
+}
+
+#if !GTK_CHECK_VERSION(3, 0, 0)
+static gboolean client_event_cb (GtkWidget *widget, GdkEventClient *event,
+                                 gpointer data)
+{
+  js_queue_cb();
   return TRUE;
 }
+#endif
+
+#if GTK_CHECK_VERSION(3, 0, 0)
+static gboolean js_queue_cb_0 (gpointer data)
+{
+  js_queue_cb();
+  return FALSE;
+}
+#endif
+
+void engine_queue_push (const char *command, se_o_t *obj, se_print_cb_t cb,
+                        void *cb_data, se_error_cb_t ecb, const char *src)
+{
+  exec_t *data = malloc (sizeof (exec_t));
+  *data = (exec_t){ g_strdup (command), obj, cb, cb_data, ecb, g_strdup (src) };
+  g_async_queue_push (js_queue, data);
+
+#if !GTK_CHECK_VERSION(3, 0, 0)
+  GdkEvent event;
+  event.client.type = GDK_CLIENT_EVENT;
+  event.client.window = gtk_widget_get_window (app);
+  event.client.send_event = TRUE;
+  event.client.message_type = GDK_NONE;
+  event.client.data_format = 8;
+  strcpy (event.client.data.b, "Javascript");
+  gdk_event_put (&event);
+#else
+  gdk_threads_add_idle (js_queue_cb_0, NULL);
+#endif
+}
 
 static char *
 load_user_startup_script (void)
@@ -353,9 +373,11 @@ reset_startup_buffer (const char *text)
 static void
 se_startup_tag_cb (GtkWidget *widget, GtkStyle *oldstyle, gpointer data)
 {
-  if (se_startup_tag)
+  if (se_startup_tag) {
+    GtkStyle *style = gtk_widget_get_style(widget);
     g_object_set (se_startup_tag, "foreground-gdk",
-		  &widget->style->text[GTK_STATE_INSENSITIVE], NULL);
+		  &style->text[GTK_STATE_INSENSITIVE], NULL);
+  }
 }
 
 static void
@@ -409,7 +431,7 @@ se_startup_response_cb (GtkWidget *widget, int response, gpointer data)
 static JSBool
 show_startup (JSContext *cx, uintN argc, jsval *vp)
 {
-  if (!GTK_WIDGET_VISIBLE (se_startup_window))
+  if (!gtk_widget_get_visible (se_startup_window))
     reset_startup_buffer (se_startup_cmds);
   window_show (se_startup_window, NULL);
   JS_SET_RVAL (cx, vp, JSVAL_VOID);
@@ -472,8 +494,8 @@ void engine_init (void)
   se_startup_textarea = gtk_text_view_new_with_buffer (se_startup_buf);
   gtk_widget_set_name (se_startup_textarea, "gxine_js_edit");
   gtk_container_add (GTK_CONTAINER (w), se_startup_textarea);
-  gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (se_startup_window)->vbox),
-			       w);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (se_startup_window))),
+                      w, TRUE, TRUE, 0);
   g_signal_connect (se_startup_textarea, "style-set",
 		    G_CALLBACK (se_startup_tag_cb), NULL);
   g_signal_connect (se_startup_buf, "insert-text",
@@ -483,16 +505,16 @@ void engine_init (void)
   g_async_queue_ref (js_queue);
 }
 
-static gboolean
-engine_startup_cb (gpointer data)
+void
+engine_startup (void)
 {
   /* note: order of actions is significant */
   queue_available = TRUE;
+#if !GTK_CHECK_VERSION(3, 0, 0)
   g_signal_connect (G_OBJECT (app), "client-event",
-		    G_CALLBACK (js_queue_cb), NULL);
-  js_queue_cb (NULL, NULL, NULL);
-  gxine_init_done ();
-  return FALSE;
+                    G_CALLBACK (client_event_cb), NULL);
+#endif
+  js_queue_cb ();
 }
 
 void
@@ -507,11 +529,12 @@ engine_startup_script (void)
 		   name, strerror (errno));
   else if (file)
   {
+    GtkStyle *style = gtk_widget_get_style (se_startup_textarea);
     se_eval_ext (gse, file, NULL, NULL, NULL, se_error_cb,
 		 _("Startup script (system)"));
     se_startup_tag = gtk_text_buffer_create_tag (se_startup_buf, "global",
 		"editable", FALSE, "foreground-gdk",
-		&se_startup_textarea->style->text[GTK_STATE_INSENSITIVE],
+                &style->text[GTK_STATE_INSENSITIVE],
 		"scale", 0.9, NULL);
     gtk_text_buffer_get_start_iter (se_startup_buf, &pos);
     gtk_text_buffer_insert_with_tags (se_startup_buf, &pos, file, -1,
@@ -524,8 +547,6 @@ engine_startup_script (void)
 
   se_startup_cmds = load_user_startup_script ();
   se_eval (gse, se_startup_cmds, NULL, NULL, NULL, _("Startup script"));
-
-  gxine_init_add ((GtkFunction) engine_startup_cb, NULL);
 }
 
 void
