@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2007 the xine project
+ * Copyright (C) 2001-2017 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,16 +72,14 @@
 #endif
 
 #include <gdk/gdk.h>
-#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkmain.h>
 #if defined(GDK_WINDOWING_X11)
 #include <gdk/gdkx.h>
 #elif defined(GDK_WINDOWING_WIN32)
 #include <windows.h>
 #include <gdk/gdkwin32.h>
 #endif
+
 #include <xine.h>
 /* for get_capabilities() */
 #include <xine/video_out.h>
@@ -98,8 +96,8 @@ extern int XShmGetEventBase(Display *);
 #endif
 #endif
 
-static void gtk_video_class_init    (GtkVideoClass   *klass);
-static void gtk_video_instance_init (GtkVideo        *gvideo);
+static void gtk_video_class_init    (gpointer,        gpointer);
+static void gtk_video_instance_init (GTypeInstance *, gpointer);
 
 static void gtk_video_finalize      (GObject        *object);
 static void gtk_video_realize       (GtkWidget      *widget);
@@ -111,7 +109,7 @@ static void gtk_video_size_request (GtkWidget      *widget,
 				    GtkRequisition *requisition);
 */
 static void gtk_video_size_allocate (GtkWidget      *widget,
-				     GtkAllocation  *allocation);
+                                     GtkAllocation  *allocation);
 
 static gboolean gtv_unblank_screen (GtkVideo *);
 static gboolean gtv_update_resize_factor (GtkVideo *);
@@ -133,7 +131,7 @@ static pthread_mutex_t resize_add_lock = PTHREAD_MUTEX_INITIALIZER;
 #define RESIZE_ADD(resize) \
   do { \
     pthread_mutex_lock (&resize_add_lock); \
-    if (!gtv->priv->resizing && GTK_WIDGET_REALIZED (&gtv->widget)) \
+    if (!gtv->priv->resizing && gtk_widget_get_realized (&gtv->widget)) \
       gtv->priv->resizing = \
 	g_idle_add_full (G_PRIORITY_HIGH_IDLE, \
 			 (GSourceFunc) ((resize) \
@@ -153,7 +151,7 @@ struct gtk_video_private_s {
   xine_t                  *xine;
   xine_stream_t           *stream;
   xine_post_out_t         *out;
-  char                    *video_driver_id;
+  const char              *video_driver_id;
   xine_video_port_t       *video_port;
 
 #if defined(GDK_WINDOWING_X11)
@@ -249,18 +247,19 @@ static int shift = 0; /* Shift key state */
 
 static inline GdkDisplay *gtv_get_display (GtkVideo *gtv)
 {
-  return gdk_drawable_get_display ((GdkDrawable *)gtv->widget.window);
+  return gtk_widget_get_display (&gtv->widget);
 }
 
 static inline GdkScreen *gtv_get_screen (GtkVideo *gtv)
 {
-  return gdk_drawable_get_screen ((GdkDrawable *)gtv->widget.window);
+  return gtk_widget_get_screen (&gtv->widget);
 }
 
 #ifdef HAVE_X11
 static inline Display *gtv_get_xdisplay (GtkVideo *gtv)
 {
-  return GDK_WINDOW_XDISPLAY (gtv->widget.window);
+  GdkWindow *window = gtk_widget_get_window (&gtv->widget);
+  return GDK_WINDOW_XDISPLAY (window);
 }
 
 static inline Screen *gtv_get_xscreen (GtkVideo *gtv)
@@ -285,9 +284,10 @@ static gboolean gtv_test_inhibit_blank (GtkVideo *gtv)
    * && !logo && !paused
    */
   return (is_fullscreen (gtv->priv)
-	  || (gtv->priv->playing_no_blank && GTK_WIDGET_DRAWABLE (&gtv->widget)
+          || (gtv->priv->playing_no_blank
+              && gtk_widget_is_drawable (&gtv->widget)
 	      && xine_get_status (gtv->priv->stream) == XINE_STATUS_PLAY
-	      && (gdk_window_get_state (gtk_widget_get_toplevel (&gtv->widget)->window) & 3) == 0))
+              && (gdk_window_get_state (gtk_widget_get_window (gtk_widget_get_toplevel (&gtv->widget))) & 3) == 0))
 	 && !playlist_showing_logo ()
 	 && xine_get_param (gtv->priv->stream, XINE_PARAM_SPEED);
 }
@@ -418,22 +418,17 @@ gtv_finalise_dbus (gtk_video_private_t *priv)
 #define gtv_finalise_dbus(PRIV)
 #endif
 
-GtkType gtk_video_get_type (void)
+GType gtk_video_get_type (void)
 {
-  static GtkType gtk_video_type = 0;
+  static GType gtk_video_type = 0;
 
   if (!gtk_video_type)
   {
     static const GTypeInfo gtk_video_info = {
-      sizeof (GtkVideoClass),
-      (GBaseInitFunc) NULL,
-      (GBaseFinalizeFunc) NULL,
-      (GClassInitFunc) gtk_video_class_init,
-      (GClassFinalizeFunc) NULL,
-      NULL /* class_data */,
-      sizeof (GtkVideo),
-      0 /* n_preallocs */,
-      (GInstanceInitFunc) gtk_video_instance_init,
+      .class_size       = sizeof (GtkVideoClass),
+      .class_init       = gtk_video_class_init,
+      .instance_size    = sizeof (GtkVideo),
+      .instance_init    = gtk_video_instance_init,
     };
 
     gtk_video_type = g_type_register_static (GTK_TYPE_WIDGET,
@@ -449,7 +444,8 @@ static void gtv_set_pointer (GtkVideo *gtv, gboolean state)
   {
     gtv->priv->pointer_visible = state;
     gtv->priv->pointer_shape_old = gtv->priv->pointer_shape;
-    gdk_window_set_cursor(gtv->widget.window, gtv->priv->pointer[state ? gtv->priv->pointer_shape + 1 : 0]);
+    gdk_window_set_cursor(gtk_widget_get_window (&gtv->widget),
+                          gtv->priv->pointer[state ? gtv->priv->pointer_shape + 1 : 0]);
   }
 }
 
@@ -520,15 +516,15 @@ static gboolean key_snoop_cb (GtkWidget *w, GdkEventKey *e, gpointer d)
 
 #endif
 
-static void gtk_video_class_init (GtkVideoClass *class)
+static void gtk_video_class_init (gpointer g_class, gpointer class_data)
 {
   GObjectClass    *object_class;
   GtkWidgetClass  *widget_class;
 
-  object_class = (GObjectClass *) class;
-  widget_class = (GtkWidgetClass *) class;
+  object_class = (GObjectClass *) g_class;
+  widget_class = (GtkWidgetClass *) g_class;
 
-  parent_class = gtk_type_class (gtk_widget_get_type ());
+  parent_class = g_type_class_ref (GTK_TYPE_WIDGET);
 
   /* GtkWidget */
   widget_class->realize       = gtk_video_realize;
@@ -536,7 +532,11 @@ static void gtk_video_class_init (GtkVideoClass *class)
   widget_class->unrealize     = gtk_video_unrealize;
   widget_class->size_allocate = gtk_video_size_allocate;
   /*widget_class->size_request  = gtk_video_size_request;*/
+#if !GTK_CHECK_VERSION(3, 0, 0)
   widget_class->expose_event  = (typeof (widget_class->expose_event)) gtk_true;
+#else
+  widget_class->draw  = (typeof (widget_class->draw)) gtk_true;
+#endif
 
   /* GObject */
   object_class->set_property  = NULL;
@@ -558,10 +558,18 @@ static void gtk_video_class_init (GtkVideoClass *class)
 #endif
 }
 
-static void gtk_video_instance_init (GtkVideo *this)
+static void gtk_video_instance_init (GTypeInstance *g_instance, gpointer g_class)
 {
+  GtkVideo *this = (GtkVideo *)g_instance;
+#if !GTK_CHECK_VERSION(2, 24, 0)
   this->widget.requisition.width  = GTK_VIDEO_MIN_WIDTH;
   this->widget.requisition.height = GTK_VIDEO_MIN_HEIGHT;
+#else
+  g_object_set (this,
+                "width-request", GTK_VIDEO_MIN_WIDTH,
+                "height-request", GTK_VIDEO_MIN_HEIGHT,
+                NULL);
+#endif
 }
 
 static void gtk_video_finalize (GObject *object)
@@ -575,20 +583,22 @@ static void gtk_video_finalize (GObject *object)
 static void gtv_do_resize (GtkVideo *gtv, const GtkRequisition *size)
 {
   gtk_widget_set_size_request (&gtv->widget, size->width, size->height);
-  if (GTK_WIDGET_REALIZED(&gtv->widget) && gtv->widget.parent)
+  if (gtk_widget_get_realized(&gtv->widget) && gtk_widget_get_parent(&gtv->widget))
   {
     GdkGeometry geom;
     GtkWindow *toplevel = GTK_WINDOW (gtk_widget_get_toplevel (&gtv->widget));
-    if (toplevel->bin.child == &gtv->widget)
+    GtkWidget *child    = gtk_bin_get_child (GTK_BIN (toplevel));
+
+    if (child == &gtv->widget)
     {
       geom.min_width = size->width;
       geom.min_height = size->height;
       gtk_window_set_geometry_hints (toplevel, &gtv->widget, &geom,
 				     GDK_HINT_MIN_SIZE);
     }
-    gtk_window_resize ((GtkWindow *)gtk_widget_get_toplevel (&gtv->widget),
+    gtk_window_resize (GTK_WINDOW (gtk_widget_get_toplevel (&gtv->widget)),
 		       size->width, size->height); /* note controlling geometry */
-    if (toplevel->bin.child == &gtv->widget)
+    if (child == &gtv->widget)
     {
       geom.min_aspect = geom.max_aspect = (double)size->width / (double)size->height;
       gtk_window_set_geometry_hints (toplevel, &gtv->widget, &geom,
@@ -637,8 +647,10 @@ static void dest_size_cb (void *gv_gen,
   }
   else
   {
-    *dest_width  = gv->widget.allocation.width;
-    *dest_height = gv->widget.allocation.height;
+    GtkAllocation allocation = { 0 };
+    gtk_widget_get_allocation (&gv->widget, &allocation);
+    *dest_width  = allocation.width;
+    *dest_height = allocation.height;
   }
 
   *dest_pixel_aspect = priv->display_ratio;
@@ -657,7 +669,7 @@ static gboolean gtv_idle_resize (GtkVideo *gtv, gboolean resize)
   logprintf ("gtkvideo: idle resize to %d x %d (factor %lf)\n",
 	     priv->size.width, priv->size.height, factor);
 
-  if (is_fullscreen (priv) || !GTK_WIDGET_VISIBLE (&gtv->widget))
+  if (is_fullscreen (priv) || !gtk_widget_get_visible (&gtv->widget))
   {
     priv->pending_resize = TRUE;
     logprintf ("gtkvideo: idle deferred (full-screen mode or widget hidden)\n");
@@ -771,7 +783,7 @@ static void frame_output_cb (void *gv_gen,
       if (first ||
 	  video_width != priv->size.width || video_height != priv->size.height)
       {
-	if (is_fullscreen (priv) || !GTK_WIDGET_VISIBLE (&gtv->widget))
+	if (is_fullscreen (priv) || !gtk_widget_get_visible (&gtv->widget))
 	{
 	  priv->size = (GtkRequisition){ video_width, video_height };
 	  priv->pending_resize = TRUE;
@@ -793,10 +805,13 @@ static void frame_output_cb (void *gv_gen,
   }
   else
   {
+    GtkAllocation allocation = { 0 };
+    gtk_widget_get_allocation (&gtv->widget, &allocation);
+
     *win_x = priv->pos.width;
     *win_y = priv->pos.height;
-    *dest_width  = gtv->widget.allocation.width;
-    *dest_height = gtv->widget.allocation.height;
+    *dest_width  = allocation.width;
+    *dest_height = allocation.height;
   }
 
   *dest_pixel_aspect = priv->display_ratio;
@@ -833,7 +848,7 @@ static xine_video_port_t *load_video_out_driver (GtkVideo *this)
   xine_video_port_t       *video_port = NULL;
   gtk_video_private_t     *priv = this->priv;
   GdkScreen               *gdkscreen;
-  char                    *desc;
+  gchar                   *desc;
 
 #ifdef HAVE_X11
 #ifndef HAVE_XCB
@@ -910,7 +925,7 @@ static xine_video_port_t *load_video_out_driver (GtkVideo *this)
 				   N_("display width in mm\n(0 = autodetect)"), desc,
 				   10, NULL, NULL);
   res_h = res_h * 1000 / (mm ? mm : mm_h);
-  free (desc);
+  g_free (desc);
 
   desc = g_strdup_printf (_("The reported value is %dmm."), mm_v);
   mm = xine_config_register_range (priv->xine, "video.display_height",
@@ -918,16 +933,16 @@ static xine_video_port_t *load_video_out_driver (GtkVideo *this)
 				   N_("display height in mm\n(0 = autodetect, -1 = assume square pixels)"), desc,
 				   10, NULL, NULL);
   res_v = mm < 0 ? res_h : (res_v * 1000 / (mm ? mm : mm_v));
-  free (desc);
+  g_free (desc);
 
   priv->display_ratio   = (res_h <= 0.0 || res_v <= 0.0) ? 1 : res_v / res_h;
   if (fabs(priv->display_ratio - 1.0) < 0.01)
     priv->display_ratio   = 1.0;
 
-  char **choices = get_driver_ids (gxine_list_video_output_plugins (priv->xine));
+  const char * const *choices = gxine_list_video_output_plugins (priv->xine);
   /* try to init video with stored information */
   int i = xine_config_register_enum (priv->xine,
-				     "video.driver", 0, choices,
+                                     "video.driver", 0, (char **)choices,
 				     N_("video driver to use"),
 				     NULL, 10, NULL, NULL);
   video_driver_id = priv->video_driver_id ? : choices[i];
@@ -1024,7 +1039,7 @@ static GdkFilterReturn xevent_filter_cb (XEvent *const xev, GdkEvent *const ev,
 {
   gtk_video_private_t *const priv = ((GtkVideo *) gtv)->priv;
 
-  if (xev->xany.window != GDK_WINDOW_XWINDOW (gtv->widget.window))
+  if (xev->xany.window != GDK_WINDOW_XID (gtk_widget_get_window (&gtv->widget)))
     return GDK_FILTER_CONTINUE;
 
   switch (xev->type)
@@ -1146,8 +1161,9 @@ static GdkFilterReturn xevent_filter_cb (XEvent *const xev, GdkEvent *const ev,
 #endif /* XTEST && LOG */
 
   default:
-    if (xev->type < LASTEvent)
+    if (xev->type < LASTEvent) {
       logprintf ("gtkvideo: received event %d\n", xev->type);
+    }
   }
 
   return GDK_FILTER_CONTINUE;
@@ -1157,8 +1173,11 @@ static GdkFilterReturn xevent_filter_cb (XEvent *const xev, GdkEvent *const ev,
 static gboolean configure_cb (GtkWidget *widget, GdkEventConfigure *event,
 			      GtkVideo *this)
 {
-  this->priv->pos.width = event->x + this->widget.allocation.width;
-  this->priv->pos.height = event->y + this->widget.allocation.height;
+  GtkAllocation allocation = { 0 };
+  gtk_widget_get_allocation (&this->widget, &allocation);
+
+  this->priv->pos.width = event->x + allocation.width;
+  this->priv->pos.height = event->y + allocation.height;
 #if !defined(HAVE_X11) && defined(WIN32)
   xine_port_send_gui_data (this->priv->video_port, GUI_WIN32_MOVED_OR_RESIZED, NULL);
 #endif
@@ -1167,7 +1186,10 @@ static gboolean configure_cb (GtkWidget *widget, GdkEventConfigure *event,
 
 static void show_cb (GtkWidget *w, gpointer d)
 {
-  gtk_widget_set_size_request (w, w->allocation.width, w->allocation.height);
+  GtkAllocation allocation = { 0 };
+  gtk_widget_get_allocation (w, &allocation);
+
+  gtk_widget_set_size_request (w, allocation.width, allocation.height);
   gtk_video_allow_shrink (GTK_VIDEO (w));
 }
 
@@ -1237,16 +1259,15 @@ static void gtv_atom_cardinal (Display *display, Window window,
 const GtkRequisition *gtk_video_get_fullscreen_geometry (GtkVideo *gtv)
 {
   GdkScreen *screen = gtv_get_screen (gtv);
+  GdkRectangle geom;
+  int monitor = gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (&gtv->widget));
+  gdk_screen_get_monitor_geometry (screen, monitor, &geom);
 
 #if defined(HAVE_X11) && defined(HAVE_XINERAMA)
   Display *display = gtv_get_xdisplay (gtv);
 
   if (XineramaIsActive (display))
   {
-    GdkRectangle geom;
-    int monitor = gdk_screen_get_monitor_at_window (screen, gtv->widget.window);
-    gdk_screen_get_monitor_geometry (screen, monitor, &geom);
-
     int count;
     XineramaScreenInfo *xinerama = XineramaQueryScreens (display, &count);
 
@@ -1272,10 +1293,6 @@ const GtkRequisition *gtk_video_get_fullscreen_geometry (GtkVideo *gtv)
   else
 #endif
   {
-    GdkRectangle geom;
-    gdk_screen_get_monitor_geometry (screen, gdk_screen_get_monitor_at_window
-					       (screen, gtv->widget.window),
-				     &geom);
     gtv->priv->fullscreen.width = geom.width;
     gtv->priv->fullscreen.height = geom.height;
   }
@@ -1313,8 +1330,6 @@ static void gtk_video_realize (GtkWidget *widget)
 #if defined(HAVE_X11) && !defined(USE_GDK_WINDOW)
   Display	      *xdisplay;
   XColor               black_pixel;
-
-  static char bm_no_data[] = { 0,0,0,0, 0,0,0,0 };
 #endif
 
   g_return_if_fail(widget != NULL);
@@ -1324,7 +1339,7 @@ static void gtk_video_realize (GtkWidget *widget)
   priv = this->priv;
 
   /* set realized flag */
-  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+  gtk_widget_set_realized (widget, TRUE);
 
   display = gdk_display_open (gdk_display_get_name (gdk_display_get_default ()));
   if (!display)
@@ -1342,29 +1357,39 @@ static void gtk_video_realize (GtkWidget *widget)
   /*
    * create our own video window
    */
+  GtkAllocation allocation = { 0 };
+  gtk_widget_get_allocation (widget, &allocation);
 
   black_pixel.pixel = BlackPixel (xdisplay, GDK_SCREEN_XNUMBER (screen));
 
   priv->video_window = XCreateSimpleWindow (xdisplay,
-					    GDK_WINDOW_XWINDOW (gtk_widget_get_parent_window(widget)),
-					    widget->allocation.x,
-					    widget->allocation.y,
-					    widget->allocation.width,
-					    widget->allocation.height, 0,
+					    GDK_WINDOW_XID (gtk_widget_get_parent_window(widget)),
+                                            allocation.x,
+                                            allocation.y,
+                                            allocation.width,
+                                            allocation.height, 0,
 					    black_pixel.pixel, black_pixel.pixel);
 
   /* not sure why this is needed in some situations... */
   XEvent event;
   XCheckWindowEvent (xdisplay, priv->video_window, -1, &event);
 
-  widget->window = gdk_window_foreign_new_for_display
-		     (display, priv->video_window);
+#  if !GTK_CHECK_VERSION(2, 24, 0)
+  GdkWindow *new_window = gdk_window_foreign_new_for_display (display, priv->video_window);
+#  else
+  GdkWindow *new_window = gdk_x11_window_foreign_new_for_display (display, priv->video_window);
+#  endif
+  gtk_widget_set_window (widget, new_window);
+
 #else /* HAVE_X11 && !USE_GDK_WINDOW */
+  GtkAllocation allocation = { 0 };
+  gtk_widget_get_allocation (widget, &allocation);
+
   GdkWindowAttr attrs = {
-       x: widget->allocation.x,
-       y: widget->allocation.y,
-       width: widget->allocation.width,
-       height: widget->allocation.height,
+       x: allocation.x,
+       y: allocation.y,
+       width: allocation.width,
+       height: allocation.height,
        wclass: GDK_INPUT_OUTPUT,
        window_type: GDK_WINDOW_CHILD,
        type_hint: GDK_WINDOW_TYPE_HINT_NORMAL,
@@ -1375,20 +1400,22 @@ static void gtk_video_realize (GtkWidget *widget)
   int i;
 
   memset(&black, 0, sizeof black);
-  widget->window = gdk_window_new(gtk_widget_get_parent_window(widget), &attrs, GDK_WA_X | GDK_WA_Y | GDK_WA_TYPE_HINT);
+  GdkWindow *new_window = gdk_window_new(gtk_widget_get_parent_window(widget), &attrs, GDK_WA_X | GDK_WA_Y | GDK_WA_TYPE_HINT);
+  gtk_widget_set_window (widget, new_window);
 #if defined(GDK_WINDOWING_X11)
-  priv->video_window = GDK_WINDOW_XWINDOW(widget->window);
+  priv->video_window = GDK_WINDOW_XID( gtk_widget_get_window (widget));
 #elif defined(GDK_WINDOWING_WIN32)
-  priv->video_window = GDK_WINDOW_HWND(widget->window);
+  priv->video_window = GDK_WINDOW_HWND( gtk_widget_get_window (widget));
 #endif
 #endif /* HAVE_X11 && !USE_GDK_WINDOW */
 
-  if (!widget->window)
+  GdkWindow *window = gtk_widget_get_window(widget);
+  if (!window)
   {
     g_printerr (_("gtkvideo: couldn't get a GDK handle for the video window!\n"));
     return;
   }
-  gdk_window_set_user_data (widget->window, widget); /* allow GTK events */
+  gdk_window_set_user_data (window, widget); /* allow GTK events */
   {
     /* enforce background colour */
     static const GdkColor black = {};
@@ -1429,14 +1456,14 @@ static void gtk_video_realize (GtkWidget *widget)
   }
 #endif
 
-  gdk_window_set_events (widget->window, GDK_ALL_EVENTS_MASK);
+  gdk_window_set_events (window, GDK_ALL_EVENTS_MASK);
 #ifndef USE_GDK_WINDOW
   XSelectInput (xdisplay, priv->video_window,
 		StructureNotifyMask | ExposureMask | FocusChangeMask |
 		KeyPressMask | KeyReleaseMask | PointerMotionMask |
 		ButtonPressMask | ButtonReleaseMask | ButtonMotionMask);
 #endif
-  gdk_window_add_filter (widget->window, (GdkFilterFunc) xevent_filter_cb,
+  gdk_window_add_filter (window, (GdkFilterFunc) xevent_filter_cb,
 			 widget);
 
 #ifndef USE_GDK_WINDOW
@@ -1459,10 +1486,15 @@ static void gtk_video_realize (GtkWidget *widget)
   /*
    * create mouse pointers
    */
-
+#if !GTK_CHECK_VERSION(3, 16, 0)
   priv->pointer[0] = gdk_cursor_new(GDK_BLANK_CURSOR);
   priv->pointer[1] = gdk_cursor_new(GDK_LEFT_PTR);
   priv->pointer[2] = gdk_cursor_new(GDK_HAND1);
+#else
+  priv->pointer[0] = gdk_cursor_new_for_display(display, GDK_BLANK_CURSOR);
+  priv->pointer[1] = gdk_cursor_new_for_display(display, GDK_LEFT_PTR);
+  priv->pointer[2] = gdk_cursor_new_for_display(display, GDK_HAND1);
+#endif
   priv->pointer_visible = TRUE;
 
 #ifdef XDG_SCREENSAVER_COMMAND
@@ -1498,10 +1530,10 @@ static void gtk_video_unrealize (GtkWidget *widget)
   }
 
   /* Hide all windows */
-  if (GTK_WIDGET_MAPPED (widget))
+  if (gtk_widget_get_mapped (widget))
     gtk_widget_unmap (widget);
 
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_MAPPED);
+  gtk_widget_set_mapped (widget, FALSE);
 
   /* This destroys widget->window and unsets the realized flag */
   if (GTK_WIDGET_CLASS(parent_class)->unrealize)
@@ -1523,7 +1555,7 @@ static void gtk_video_map (GtkWidget *widget)
 
 GtkWidget *gtk_video_new (xine_t *xine, xine_stream_t *stream,
 			  xine_post_out_t *out,
-			  const gchar *video_driver_id,
+                          const char *video_driver_id,
 			  int default_width, int default_height,
 			  int button_press_mask, int button_release_mask)
 {
@@ -1550,10 +1582,7 @@ GtkWidget *gtk_video_new (xine_t *xine, xine_stream_t *stream,
   priv->frame_cb = NULL;
   priv->pointer_shape_old = -1;
 
-  if (video_driver_id)
-    priv->video_driver_id = strdup (video_driver_id);
-  else
-    priv->video_driver_id = NULL;
+  priv->video_driver_id = video_driver_id;
 
   gtv_init_dbus (gtv);
 
@@ -1579,15 +1608,19 @@ static gboolean gtv_update_resize_factor (GtkVideo *this)
   gboolean send_signal = FALSE;
   gtk_video_private_t *const priv = this->priv;
   double new_factor;
+
+  GtkAllocation allocation = { 0 };
+  gtk_widget_get_allocation (&this->widget, &allocation);
+
   if (priv->pending_resize)
     new_factor = priv->resize_factor;
   else
   {
     gboolean is_double =
       IS_DOUBLE(priv->video_size.width, priv->video_size.height);
-    new_factor = this->widget.allocation.width * 100
+    new_factor = allocation.width * 100
 		 / (double)priv->video_size.width / (1 + is_double);
-    double new_factor_V = this->widget.allocation.height * 100
+    double new_factor_V = allocation.height * 100
 			  / (double)priv->video_size.height
 			  / (1 + is_double);
     if (new_factor > new_factor_V)
@@ -1597,10 +1630,10 @@ static gboolean gtv_update_resize_factor (GtkVideo *this)
   if (priv->pending_resize
       || (round (priv->resize_factor * 64) != round (new_factor * 64)
 	  && (fabs (new_factor - priv->resize_factor)
-		>= 25 / this->widget.allocation.height
+		>= 25.0 / allocation.height
 	      || priv->old_video_size.height == priv->video_size.height)))
   {
-    if (!priv->block_next_scale_change && GTK_WIDGET_VISIBLE (&this->widget))
+    if (!priv->block_next_scale_change && gtk_widget_get_visible (&this->widget))
     {
       priv->resize_factor = new_factor;
       send_signal = TRUE;
@@ -1609,14 +1642,15 @@ static gboolean gtv_update_resize_factor (GtkVideo *this)
       priv->size = SCALE (factor);
       logprintf ("gxine: video rescaled to %lf%%\n", new_factor);
     }
-    else
+    else {
       logprintf ("blocked rescaling to %lf%%\n", new_factor);
+    }
   }
   return send_signal;
 }
 
 static void gtk_video_size_allocate (GtkWidget *widget,
-				     GtkAllocation *allocation)
+                                     GtkAllocation *p_allocation)
 {
   GtkVideo *this;
 
@@ -1625,24 +1659,28 @@ static void gtk_video_size_allocate (GtkWidget *widget,
 
   this = (GtkVideo *) widget;
 
-  if (GTK_WIDGET_REALIZED (widget))
+  if (gtk_widget_get_realized (widget))
   {
     gboolean send_signal = FALSE;
-    widget->allocation = *allocation;
+    GtkAllocation allocation = *p_allocation;
+
     if (is_fullscreen (this->priv))
     {
-      widget->allocation.width = this->priv->fullscreen.width;
-      widget->allocation.height = this->priv->fullscreen.height;
+      allocation.width  = this->priv->fullscreen.width;
+      allocation.height = this->priv->fullscreen.height;
     }
-    gdk_window_move_resize (widget->window, allocation->x, allocation->y,
-			    widget->allocation.width,
-			    widget->allocation.height);
+
+    gtk_widget_set_allocation (widget, &allocation);
+
+    gdk_window_move_resize (gtk_widget_get_window (widget),
+                            allocation.x, allocation.y,
+                            allocation.width, allocation.height);
     if (this->priv->old_video_size.width && !this->priv->resizing
 	&& !this->priv->idle_resized
 	&& !is_fullscreen (this->priv)
-	&& GTK_WIDGET_VISIBLE (widget)
-	&& widget->allocation.width > GTK_VIDEO_MIN_WIDTH
-	&& widget->allocation.height > GTK_VIDEO_MIN_HEIGHT)
+	&& gtk_widget_get_visible (widget)
+        && allocation.width > GTK_VIDEO_MIN_WIDTH
+        && allocation.height > GTK_VIDEO_MIN_HEIGHT)
       send_signal = gtv_update_resize_factor (this);
     if (!this->priv->resizing)
       this->priv->old_video_size = this->priv->video_size;
@@ -1654,9 +1692,9 @@ static void gtk_video_size_allocate (GtkWidget *widget,
 		     this->priv->resize_factor);
   }
   else
-    widget->allocation = *allocation;
+    gtk_widget_set_allocation (widget, p_allocation);
 
-  if (widget->window)
+  if (gtk_widget_get_window(widget))
   {
     /* FIXME: stop something unknown (as of 20051014) from blanking the window
      * (no map/unmap event handlers are expected)
@@ -1687,7 +1725,7 @@ void gtk_video_rescale (GtkVideo *gtv, double scale)
     priv->resize_factor = scale;
     logprintf ("rescale: size = %d,%d factor=%f\n",
 	       priv->size.width, priv->size.height, scale);
-    if (is_fullscreen (priv) || !GTK_WIDGET_VISIBLE (&gtv->widget))
+    if (is_fullscreen (priv) || !gtk_widget_get_visible (&gtv->widget))
       priv->pending_resize = TRUE;
     else
       gtv_do_rescale (gtv);
@@ -1726,7 +1764,7 @@ void gtk_video_reshow (GtkVideo *gtv)
 }
 
 void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen,
-			       GStaticRecMutex *const lock)
+                               GRecMutex *const lock)
 {
   gtk_video_private_t  *priv;
   GtkWindow	       *toplevel;
@@ -1742,7 +1780,7 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen,
   if (fullscreen == is_fullscreen (priv))
     return;
 
-  g_static_rec_mutex_lock (lock);
+  g_rec_mutex_lock (lock);
 #ifdef HAVE_X11
   display = gtv_get_xdisplay (gtv);
   XLockDisplay (display);
@@ -1751,11 +1789,14 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen,
 
   if (fullscreen)
   {
+    GtkAllocation allocation = { 0 };
+    gtk_widget_get_allocation (&gtv->widget, &allocation);
+
     priv->fullscreen_mode = fullscreen;
 
     do_pending_events (); /* there may be pending resize events... */
-    priv->old_size.width = gtv->widget.allocation.width;
-    priv->old_size.height = gtv->widget.allocation.height;
+    priv->old_size.width = allocation.width;
+    priv->old_size.height = allocation.height;
     /* just fullscreen it - beyond that, rely on window-state-event */
     gtk_window_fullscreen (toplevel);
     do_pending_events (); /* events have been generated - process them */
@@ -1802,7 +1843,7 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen,
 
   XUnlockDisplay (display);
 #endif
-  g_static_rec_mutex_unlock (lock);
+  g_rec_mutex_unlock (lock);
 }
 
 gint gtk_video_is_fullscreen (GtkVideo *gtv)
@@ -1856,11 +1897,16 @@ gtk_video_deny_shrink (GtkVideo *gtv)
     g_source_remove (gtv->priv->minimising);
     gtv->priv->minimising = 0;
   }
-  gtv->widget.requisition = gtv->priv->pending_resize
-			    ? gtv->priv->size : gtv->priv->old_size;
+
+  GtkRequisition requisition = gtv->priv->pending_resize
+                               ? gtv->priv->size
+                               : gtv->priv->old_size;
+#if !GTK_CHECK_VERSION(2, 24, 0)
+  gtv->widget.requisition = requisition;
+#endif
   g_object_set (gtv,
-		"width-request", gtv->widget.requisition.width,
-		"height-request", gtv->widget.requisition.height,
+		"width-request", requisition.width,
+		"height-request", requisition.height,
 		NULL);
 }
 
@@ -1872,8 +1918,10 @@ gtv_allow_shrink (gpointer gtv)
   if (is_fullscreen (((GtkVideo *)gtv)->priv))
     return FALSE;
 
+#if !GTK_CHECK_VERSION(2, 24, 0)
   ((GtkVideo *)gtv)->widget.requisition =
     (GtkRequisition){ GTK_VIDEO_MIN_WIDTH, GTK_VIDEO_MIN_HEIGHT };
+#endif
   g_object_set (gtv, "width-request", -1, "height-request", -1, NULL);
   return FALSE;
 }
@@ -1920,13 +1968,14 @@ static GList *gtv_create_plugins (gtk_video_private_t *priv,
 	args = NULL;
 
       logprintf ("plugin: %p %.*s\n", plugins, (int)((args?:next) - plugins), plugins);
-      if (args)
+      if (args) {
 	logprintf ("args  : %p %.*s\n", args ? args + 1 : NULL,
 		   (int)(args ? next - args - 1 : 0), args ? args + 1 : NULL);
+      }
       logprintf ("next  : %p\n", next);
 
       name = g_strndup (plugins, (args ? : next) - plugins);
-      if (is_deinterlace && !g_strcasecmp (name, "tvtime"))
+      if (is_deinterlace && !strcasecmp (name, "tvtime"))
 	priv->have_tvtime = TRUE;
       plugin = xine_post_init (priv->xine, name, 0, audio, video);
       if (plugin)
@@ -1938,7 +1987,7 @@ static GList *gtv_create_plugins (gtk_video_private_t *priv,
 	  {
 	    args = g_strndup (args + 1, next - args - 1);
 	    post_parse_set_parameters (plugin, args);
-	    free (args);
+	    g_free (args);
 	  }
 	}
 	else
@@ -1949,7 +1998,7 @@ static GList *gtv_create_plugins (gtk_video_private_t *priv,
       }
       else
 	g_printerr (_("gtkvideo: unknown post-plugin ‘%s’\n"), name);
-      free (name);
+      g_free (name);
     }
 
     if (!*next)
@@ -2392,7 +2441,7 @@ static int xdg_screensaver_poke (const char *arg, Window id)
   char id_str[16];
   const char *const argv[] = { XDG_SCREENSAVER_COMMAND, arg,
 			       id ? id_str : NULL, NULL };
-  sprintf (id_str, "%#x", (int) id);
+  snprintf (id_str, sizeof(id_str), "%#x", (int) id);
   /* screensaver name is used in "can't poke <name>" */
   return screensaver_poke (argv, "xdg-screensaver");
 }
@@ -2409,7 +2458,7 @@ static gboolean gtv_unblank_screen (GtkVideo *gtv)
     if (inhibit != !!gtv->priv->xdg_active)
     {
       Window id = inhibit
-		? gdk_x11_drawable_get_xid (gtk_widget_get_toplevel (&gtv->widget)->window)
+                ? gdk_x11_drawable_get_xid (gtk_widget_get_window (gtk_widget_get_toplevel (&gtv->widget)))
 		: gtv->priv->xdg_active;
       int ret = 0;
       if (!id || !(ret = xdg_screensaver_poke (inhibit ? "suspend" : "resume", id)))
