@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2007 the xine project
+ * Copyright (C) 2003-2017 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,7 +29,11 @@
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
+#if GTK_CHECK_VERSION(3, 0, 0)
+#include <gdk/gdkkeysyms-compat.h>
+#endif
 #include <gtk/gtk.h>
+
 #include <xine.h>
 #include <xine/xmlparser.h>
 
@@ -70,7 +74,6 @@ typedef struct {
 
 static key_binding_t editkey;
 static const char *editdesc;
-static const key_binding_t null_binding = { 0, 0, GDK_VoidSymbol, 0 };
 static key_binding_t default_binding = { N_("New binding"), "", GDK_VoidSymbol, 0 };
 
 static GtkTreeIter catch_key_iter;
@@ -186,7 +189,7 @@ void get_menu_bar_accel (GtkWidget *w, guint *keyval, GdkModifierType *mods)
 
 static gboolean is_menu_accel (GtkWidget *w, const GdkEventKey *event)
 {
-  guint keyval;
+  guint keyval = 0;
   GdkModifierType mods;
   get_menu_bar_accel (w, &keyval, &mods);
   return event->keyval == keyval &&
@@ -228,9 +231,9 @@ gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
   if (win == app && menubar)
   {
     GtkWidget *toplevel = gtk_widget_get_toplevel (menubar);
-    if (toplevel != app && GTK_WIDGET_VISIBLE (toplevel))
+    if (toplevel != app && gtk_widget_get_visible (toplevel))
     {
-      event->window = gtk_widget_get_toplevel (menubar)->window;
+      event->window = gtk_widget_get_window (gtk_widget_get_toplevel (menubar));
       event->send_event = TRUE;
       gtk_main_do_event ((GdkEvent *) event);
       return TRUE;
@@ -247,7 +250,7 @@ gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
   else if (is_menu_accel (win, event))
   {
     GtkWidget *toplevel = gtk_widget_get_toplevel (menubar);
-    if (!menubar || !GTK_WIDGET_VISIBLE (toplevel == app ? menubar : toplevel))
+    if (!menubar || !gtk_widget_get_visible (toplevel == app ? menubar : toplevel))
       goto do_popup;
   }
 
@@ -407,7 +410,7 @@ static gchar* kb2str (key_binding_t *kb)
 
   strcat (name, key);
 
-  free (key_alloc);
+  g_free (key_alloc);
   return name;
 }
 
@@ -780,9 +783,9 @@ static void do_edit_binding (void)
   gtk_text_buffer_set_text (kb_binding_command_buf, editkey.cmd, -1);
   gtk_entry_set_text (GTK_ENTRY (kb_binding_key), editkey.keyname);
 
-  char *title = g_strdup_printf (_("Keybinding: %s"), editkey.desc);
+  gchar *title = g_strdup_printf (_("Keybinding: %s"), editkey.desc);
   gtk_window_set_title (GTK_WINDOW (kb_binding_edit_dlg), title);
-  free (title);
+  g_free (title);
 
   window_show (kb_binding_edit_dlg, kb_bindings_list_dlg);
   gtk_widget_grab_focus (kb_binding_key);
@@ -932,7 +935,11 @@ static gboolean do_edit_key_cb (GtkWidget *widget, GdkEventKey *event,
     return FALSE;
 
   if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (kb_binding_quote)))
-    if (gtk_bindings_activate_event ((GtkObject *)widget, event) ||
+#if !GTK_CHECK_VERSION(3, 0, 0)
+    if (gtk_bindings_activate_event (GTK_OBJECT (widget), event) ||
+#else
+    if (gtk_bindings_activate_event (G_OBJECT (widget), event) ||
+#endif
 	gtk_window_activate_key (GTK_WINDOW (kb_binding_edit_dlg), event))
       return TRUE;
 
@@ -1050,11 +1057,11 @@ static JSBool js_xine_event (int type
     return ret; \
   }
 
-JS_XINE_EVENT (input_up, INPUT_UP);
-JS_XINE_EVENT (input_down, INPUT_DOWN);
-JS_XINE_EVENT (input_left, INPUT_LEFT);
-JS_XINE_EVENT (input_right, INPUT_RIGHT);
-JS_XINE_EVENT (input_select, INPUT_SELECT);
+JS_XINE_EVENT (input_up, INPUT_UP)
+JS_XINE_EVENT (input_down, INPUT_DOWN)
+JS_XINE_EVENT (input_left, INPUT_LEFT)
+JS_XINE_EVENT (input_right, INPUT_RIGHT)
+JS_XINE_EVENT (input_select, INPUT_SELECT)
 
 static JSBool js_input_menu (JSContext *cx, uintN argc, jsval *vp)
 {
@@ -1089,11 +1096,11 @@ static JSBool js_input_menu (JSContext *cx, uintN argc, jsval *vp)
   return JS_TRUE;
 }
 
-JS_XINE_EVENT (input_menu1, INPUT_MENU1);
-JS_XINE_EVENT (input_menu2, INPUT_MENU2);
-JS_XINE_EVENT (input_menu3, INPUT_MENU3);
-JS_XINE_EVENT (input_previous, INPUT_PREVIOUS);
-JS_XINE_EVENT (input_next, INPUT_NEXT);
+JS_XINE_EVENT (input_menu1, INPUT_MENU1)
+JS_XINE_EVENT (input_menu2, INPUT_MENU2)
+JS_XINE_EVENT (input_menu3, INPUT_MENU3)
+JS_XINE_EVENT (input_previous, INPUT_PREVIOUS)
+JS_XINE_EVENT (input_next, INPUT_NEXT)
 
 static JSBool js_keybindings_show (JSContext *cx, uintN argc, jsval *vp)
 {
@@ -1108,7 +1115,7 @@ static JSBool js_keypad_show (JSContext *cx, uintN argc, jsval *vp)
   se_log_fncall_checkinit ("keypad_show");
   if (keypad)
   {
-    if (GTK_WIDGET_VISIBLE (keypad))
+    if (gtk_widget_get_visible (keypad))
       gtk_widget_hide (keypad);
     else
       window_show (keypad, NULL);
@@ -1194,7 +1201,7 @@ int kb_xine_event_lookup (const kb_xine_event_map_t *map, const char *token)
 
 static void kb_xine_event_help (se_t *se, const kb_xine_event_map_t *map)
 {
-  char *out;
+  gchar *out;
   const kb_xine_event_id_t *id;
 
   if (!map)
@@ -1204,7 +1211,7 @@ static void kb_xine_event_help (se_t *se, const kb_xine_event_map_t *map)
   for (id = &map->id[0]; id->name[0]; ++id)
     asreprintf (&out, "%s\n  %s", out, id->name);
   se->print_cb (se->print_cb_data, "%s", out);
-  free (out);
+  g_free (out);
 }
 
 
@@ -1427,14 +1434,14 @@ void key_events_init (void)
 
   {
     static const GtkActionEntry kb_menu_data[] = {
-      { "binding", NULL,			N_("_Reload...") },
-      { "reload",  GTK_STOCK_REVERT_TO_SAVED,	N_("_Reload bindings"),			NULL,		NULL,				kb_reload_keymap },
-      { "addnew",  GTK_STOCK_ADD,		N_("_Add new default bindings"),	NULL,		NULL,				kb_add_defaults },
-      { "merge",   NULL,			N_("_Merge default bindings"),		NULL,		NULL,				kb_merge_defaults },
-      { "replace", GTK_STOCK_CLEAR,		N_("Replace with _default bindings"),	NULL,		NULL,				kb_reset_keymap },
-      { "new",	   GTK_STOCK_NEW,		NULL,					NULL,		N_("Add a key binding"),	kb_new_binding },
-      { "edit",	   GTK_STOCK_EDIT,		NULL,					"<Control>E",	N_("Edit the selected item"),	kb_edit_binding },
-      { "delete",  GTK_STOCK_DELETE,		NULL,					"Delete",	N_("Delete the selected item"),	kb_delete_binding },
+      { "binding", NULL,                        N_("_Reload...") },
+      { "reload",  GTK_STOCK_REVERT_TO_SAVED,   N_("_Reload bindings"),                NULL,           NULL,                           G_CALLBACK(kb_reload_keymap) },
+      { "addnew",  GTK_STOCK_ADD,               N_("_Add new default bindings"),       NULL,           NULL,                           G_CALLBACK(kb_add_defaults) },
+      { "merge",   NULL,                        N_("_Merge default bindings"),         NULL,           NULL,                           G_CALLBACK(kb_merge_defaults) },
+      { "replace", GTK_STOCK_CLEAR,             N_("Replace with _default bindings"),  NULL,           NULL,                           G_CALLBACK(kb_reset_keymap) },
+      { "new",     GTK_STOCK_NEW,               NULL,                                  NULL,           N_("Add a key binding"),        G_CALLBACK(kb_new_binding) },
+      { "edit",    GTK_STOCK_EDIT,              NULL,                                  "<Control>E",   N_("Edit the selected item"),   G_CALLBACK(kb_edit_binding) },
+      { "delete",  GTK_STOCK_DELETE,            NULL,                                  "Delete",       N_("Delete the selected item"), G_CALLBACK(kb_delete_binding) },
     };
     static const char kb_menu_structure[] =
       "<ui>"
@@ -1467,10 +1474,10 @@ void key_events_init (void)
     gtk_action_group_connect_accelerators (actions);
   }
 
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(kb_bindings_list_dlg)->vbox), hbox,
-		      FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(kb_bindings_list_dlg)->vbox), scrolled_window,
-		      TRUE, TRUE, 2);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (kb_bindings_list_dlg))),
+                      hbox, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (kb_bindings_list_dlg))),
+                      scrolled_window, TRUE, TRUE, 2);
 
   list_visible = FALSE;
 
@@ -1516,7 +1523,8 @@ void key_events_init (void)
   add_table_row_items (b, 2, FALSE, _("Accelerator key"),
 		       kb_binding_key, kb_binding_quote, NULL);
 
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (kb_binding_edit_dlg)->vbox) , b);
+  gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (kb_binding_edit_dlg))),
+                     b);
 
   g_object_connect (G_OBJECT (kb_binding_edit_dlg),
 	"signal::delete-event", G_CALLBACK (close_edit_cb), NULL,
