@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2007 the xine-project
+ * Copyright (C) 2001-2017 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,14 +16,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * gtk2 ui for xine
+ * gtk2 / gtk3 ui for xine
  *
  * main
  */
 
 #include "globals.h"
 #include "version.h"
-#include <xine/xineutils.h>
 
 #include <string.h>
 #include <signal.h>
@@ -42,16 +41,15 @@
 #include <getopt.h>
 #endif
 
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkkeysyms.h>
 #include <glib.h>
+#include <gdk/gdk.h>
+#include <gtk/gtk.h>
+
+#include <xine/xineutils.h>
 
 #include "desktop_integration.h"
 #include "engine.h"
 #include "player.h"
-#include "gtkvideo.h"
-#include "info_widgets.h"
 #include "wizards.h"
 #include "xml_widgets.h"
 #include "noskin_window.h"
@@ -80,9 +78,7 @@
 
 int             verbosity;
 GtkWidget      *app = NULL;
-gchar          *plugindir, *bindir, *logodir, *pixmapdir, *icondir, *miscdir, *confdir;
-gchar          *video_driver_id=NULL;
-gchar          *audio_driver_id=NULL;
+const char     *plugindir, *bindir, *logodir, *pixmapdir, *icondir, *miscdir, *confdir;
 
 #define X_INIT_THREADS() \
   if (!XInitThreads ()) \
@@ -152,16 +148,25 @@ static void splash_show (void)
   gtk_fixed_put (GTK_FIXED (fixed), img, 0, 0);
 
   text = ui_label_new_with_markup ("<span font_desc='sans 12' color='#DDDDDD'>"VERSION"</span>");
+#if GTK_CHECK_VERSION(3, 14, 0)
+  gtk_label_set_xalign (GTK_LABEL (text), 1);
+  gtk_label_set_yalign (GTK_LABEL (text), 1);
+#else
   gtk_misc_set_alignment (GTK_MISC (text), 1, 1);
+#endif
   gtk_fixed_put (GTK_FIXED (fixed), text, 0, 0);
 
   gtk_widget_show_all (splash);
   /* the widgets need to be realised before we can move the label */
+  GtkAllocation allocation_img = { 0 };
+  GtkAllocation allocation_text = { 0 };
+  gtk_widget_get_allocation (img, &allocation_img);
+  gtk_widget_get_allocation (text, &allocation_text);
   gtk_fixed_move (GTK_FIXED (fixed), text,
-		  img->allocation.width - text->allocation.width,
-		  img->allocation.height - text->allocation.height);
+                  allocation_img.width  - allocation_text.width,
+                  allocation_img.height - allocation_text.height);
 
-  gtk_timeout_add (4000, splash_destroy, splash);
+  g_timeout_add (4000, splash_destroy, splash);
 
   do_pending_events ();
   gdk_threads_leave();
@@ -222,8 +227,8 @@ static gboolean post_init_configure (void)
 
 static gboolean main_start_play (gpointer data)
 {
-  if (gxine_init_count)
-    return TRUE;
+  //if (gxine_init_count)
+  //  return TRUE;
   playlist_play (*(int *)data);
   return FALSE;
 }
@@ -247,16 +252,16 @@ static void __attribute__ ((noreturn)) watchdog_timeout (int sig)
 
 /* Our own locking for GDK */
 
-static GStaticRecMutex gdk_lock = G_STATIC_REC_MUTEX_INIT;
+static GRecMutex gdk_lock;
 
 static void gxine_gdk_lock (void)
 {
-  g_static_rec_mutex_lock (&gdk_lock);
+  g_rec_mutex_lock (&gdk_lock);
 }
 
 static void gxine_gdk_unlock (void)
 {
-  g_static_rec_mutex_unlock (&gdk_lock);
+  g_rec_mutex_unlock (&gdk_lock);
 }
 
 /* Faults */
@@ -282,10 +287,10 @@ static void __attribute__ ((noreturn)) fault_handler (int sig)
 
 /* Main */
 
-static char *
+static const char *
 getdir (const char *env, const char *dir)
 {
-  return getenv (env) ? : g_strconcat (dir, NULL);
+  return getenv (env) ? : dir;
 }
 
 static void
@@ -310,6 +315,8 @@ static void config_change (gpointer data, gpointer user)
   int i;
   char *text = data;
   char *equals = strchr (text, '=');
+  if (!equals)
+    return;
   *equals = 0;
 
   if (xine_config_lookup_entry (xine, text, &entry))
@@ -355,6 +362,87 @@ static void config_change (gpointer data, gpointer user)
   *equals = '=';
 }
 
+#if !GTK_CHECK_VERSION(3, 0, 0)
+static void strv_add_uniq(gchar **v, const char *s)
+{
+  size_t i;
+  for (i = 0; v[i]; i++)
+    if (!strcmp(v[i], s))
+      return;
+  v[i] = strdup(s);
+}
+
+static void add_gtkrc_files(void)
+{
+  const gchar  *home = g_get_home_dir ();
+  size_t        homelen = strlen (home);
+  gchar       **file = gtk_rc_get_default_files ();
+  guint         length = file ? g_strv_length (file) : 0;
+  guint         i;
+
+  /* deep-copy the gtkrc filenames, allocating space for two more */
+  gchar       **copy = calloc (length + 3, sizeof (gchar *));
+  for (i = 0; file && file[i]; ++i)
+    strv_add_uniq(copy, file[i]);
+  length = g_strv_length (copy);
+
+  /* find the index of the first *user* gtkrc filename */
+  for (i = 0; copy[i]; ++i)
+    if (!strncmp (copy[i], home, homelen) && copy[i][homelen] == '/')
+      break;
+  /* insert our system gtkrc filename before it (strdup for convenience) */
+  if (i < length)
+    memmove (copy + i + 1, copy + i, (length - i) * sizeof (gchar *));
+  copy[i] = g_build_filename (confdir, FILE_GTKRC, NULL);
+  /* now append our user gtkrc, terminate the list and tell GTK+ about it */
+  copy[length + 1] = get_config_filename (FILE_GTKRC);
+  copy[length + 2] = NULL;
+  gtk_rc_set_default_files (copy);
+  /* free the list */
+  g_strfreev (copy);
+}
+#endif /* GTK2 */
+
+#if GTK_CHECK_VERSION(3, 0, 0)
+static void load_css_file(const char *path, guint priority)
+{
+  GtkCssProvider *css_provider;
+  GError *gerror = NULL;
+  struct stat st;
+
+  if (stat (path, &st))
+    return;
+
+  css_provider = gtk_css_provider_new();
+  gtk_css_provider_load_from_path(css_provider, path, &gerror);
+
+  if (gerror) {
+    fprintf (stderr, "Error loading styles from %s:\n\t%s\n", path, gerror->message);
+    g_error_free (gerror);
+    g_object_unref (css_provider);
+    return;
+  }
+
+  gtk_style_context_add_provider_for_screen (gdk_display_get_default_screen (gdk_display_get_default ()),
+                                             GTK_STYLE_PROVIDER (css_provider),
+                                             priority);
+}
+
+static void load_gtk_css(void)
+{
+  gchar *path;
+
+  path = g_build_filename (confdir, FILE_GTKCSS, NULL);
+  load_css_file (path, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  g_free (path);
+
+  path = get_config_filename (FILE_GTKCSS);
+  load_css_file (path, GTK_STYLE_PROVIDER_PRIORITY_USER);
+  g_free (path);
+}
+#endif /* GTK3 */
+
+gpointer cssprovider;
 int main(int argc, char* argv[])
 {
   gboolean enqueue, autoplay, fullscreen;
@@ -366,8 +454,13 @@ int main(int argc, char* argv[])
   xine_cfg_entry_t entry;
   GPtrArray *cfg_changes;
   gboolean migrated_config_dir = FALSE;
+  const char *video_driver_id = NULL;
+  const char *audio_driver_id = NULL;
 
-  g_thread_init (NULL); /* done early for glib >= 2.13 */
+#if !defined(USE_SUN_XLIB_WORKAROUND) && defined(HAVE_X11)
+  /* start using X... */
+  X_INIT_THREADS ();
+#endif
 
 #ifndef GLIB_USES_SYSTEM_MALLOC
   GMemVTable vtab = { xmalloc, xrealloc, free, NULL, NULL, NULL };
@@ -391,29 +484,9 @@ int main(int argc, char* argv[])
   confdir   = getdir ("GXINE_CONFDIR", GXINE_CONFDIR);
 
   /* set up our gtkrc stuff */
-  {
-    gchar **file = gtk_rc_get_default_files ();
-    const gchar *home = g_get_home_dir ();
-    guint homelen = strlen (home);
-    guint i, length = g_strv_length (file);
-    /* deep-copy the gtkrc filenames, allocating space for two more */
-    gchar **copy = calloc (length + 3, sizeof (gchar *));
-    for (i = 0; file[i]; ++i)
-      copy[i] = strdup (file[i]);
-    /* find the index of the first *user* gtkrc filename */
-    for (i = 0; file[i]; ++i)
-      if (!strncmp (file[i], home, homelen) && file[i][homelen] == '/')
-	break;
-    /* insert our system gtkrc filename before it (strdup for convenience) */
-    memmove (copy + i + 1, copy + i, (length - i) * sizeof (gchar *));
-    copy[i] = g_build_filename (confdir, FILE_GTKRC, NULL);
-    /* now append our user gtkrc, terminate the list and tell GTK+ about it */
-    copy[length + 1] = get_config_filename (FILE_GTKRC);
-    copy[length + 2] = NULL;
-    gtk_rc_set_default_files (copy);
-    /* free the list */
-    g_strfreev (copy);
-  }
+#if !GTK_CHECK_VERSION(3, 0, 0)
+  add_gtkrc_files();
+#endif
 
   /*
    * init glib/gdk/gtk thread safe/aware
@@ -436,6 +509,12 @@ int main(int argc, char* argv[])
   gdk_threads_set_lock_functions (gxine_gdk_lock, gxine_gdk_unlock);
   gdk_threads_init ();
 
+#if GTK_CHECK_VERSION(3,0,0)
+    /* gxine currently assumes that Gtk means X11, so it cannot use native Gtk
+     * support for other backends such as Wayland. */
+    gdk_set_allowed_backends ("x11,win32");
+#endif
+
   if (!gtk_parse_args (&argc, &argv))
     return 2;
 
@@ -593,11 +672,6 @@ to the extent permitted by law.\n"),
   if (!no_connect)
     gxine_try_remote (argc, argv, enqueue, autoplay);
 
-#if !defined(HAVE_XCB) && !defined(USE_SUN_XLIB_WORKAROUND) && defined(HAVE_X11)
-  /* start using X... */
-  X_INIT_THREADS ();
-#endif
-
   struct stat st;
   char *oldconfig = g_build_filename (g_get_home_dir (), ".gxine", NULL);
   char *newconfig = get_config_filename (NULL);
@@ -644,6 +718,14 @@ to the extent permitted by law.\n"),
   }
 #endif
 
+  /*
+   * Load GTK3 css styles
+   */
+
+#if GTK_CHECK_VERSION(3, 0, 0)
+  load_gtk_css();
+#endif
+
   /*
    * init xine, set up script engine
    */
@@ -659,7 +741,7 @@ to the extent permitted by law.\n"),
     { "gui.fullscreen_toolbar", "gui.fullscreen_mode.toolbar" },
     { "gui.windowedmode_separate_toolbar", "gui.windowed_mode.separate_toolbar" },
     { "gui.windowedmode_unblank", "gui.windowed_mode.unblank" },
-    {}
+    { NULL, NULL }
   };
   xine_config_set_translation_user (gxine_config_xlate);
 
@@ -711,7 +793,7 @@ to the extent permitted by law.\n"),
    * set up player and some UI basics
    */
 
-  player_init ();
+  player_init (audio_driver_id);
   ui_init ();
 
   gtk_window_set_default_icon_name (GXINE_LOGO);
@@ -725,9 +807,9 @@ to the extent permitted by law.\n"),
 
   do_pending_events ();
   gtk_window_set_auto_startup_notification (TRUE);
-  noskin_main_init (geometry, fullscreen);
+  noskin_main_init (video_driver_id, geometry, fullscreen);
 
-#if !defined(HAVE_XCB) && defined(USE_SUN_XLIB_WORKAROUND) && defined(HAVE_X11)
+#if defined(USE_SUN_XLIB_WORKAROUND) && defined(HAVE_X11)
   /* call XInitThreads after the window opens to avoid deadlock in libXi */
   X_INIT_THREADS ();
 #endif
@@ -771,7 +853,6 @@ to the extent permitted by law.\n"),
   post_init_configure ();
   initialised = TRUE;
 
-  gxine_init_add ((GtkFunction) ui_post_init, NULL);
   playlist_logo ((gpointer) 1);
 
   /*
@@ -802,6 +883,11 @@ to the extent permitted by law.\n"),
   if (cmd)
     se_eval (gse, cmd, NULL, NULL, NULL, "--command");
 
+  engine_startup ();
+  noskin_post_init ();
+  noskin_main_post_init (fullscreen);
+  ui_post_init ();
+
   //gdk_threads_enter();
   gtk_main();
   //gdk_threads_leave();
