@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2007 the xine-project
+ * Copyright (C) 2002-2017 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,9 +28,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
+
 #include <glib.h>
+#include <gdk/gdk.h>
+#include <gtk/gtk.h>
 
 #include "ui.h"
 #include "utils.h"
@@ -63,7 +64,7 @@ static void mm_add_at_selected (play_item_t *play_item);
 
 static inline const char *peek_title (GtkTreeIter *iter)
 {
-  static GValue v = {};
+  static GValue v = G_VALUE_INIT;
   if (G_IS_VALUE (&v))
     g_value_unset (&v);
   gtk_tree_model_get_value (mm_model, iter, COLUMN_TITLE, &v);
@@ -72,7 +73,7 @@ static inline const char *peek_title (GtkTreeIter *iter)
 
 static inline play_item_t *peek_play_item (GtkTreeIter *iter)
 {
-  GValue v = {};
+  GValue v = G_VALUE_INIT;
   gtk_tree_model_get_value (mm_model, iter,
 			    COLUMN_PLAY_ITEM, &v);
   void *p = g_value_peek_pointer (&v);
@@ -106,11 +107,11 @@ static int gen_actions_sub (GtkUIManager *ui, GtkActionGroup *actions,
     const char *id = peek_title (iter);
     if (id)
     {
-      char *name = g_strdup_printf ("_mediamark_%d", count++);
+      gchar *name = g_strdup_printf ("_mediamark_%d", count++);
       GtkAction *action = gtk_action_new (name, id, NULL, NULL);
 
       gtk_action_group_add_action (actions, action);
-      free (name);
+      g_free (name);
       play_item_t *item = peek_play_item (iter);
       if (item)
 	g_signal_connect (G_OBJECT(action), "activate",
@@ -156,9 +157,9 @@ static int gen_items (GtkUIManager *ui, const char *path, GtkTreeIter *iter,
 			       GTK_UI_MANAGER_MENU, FALSE);
 	if (gtk_tree_model_iter_children (mm_model, &children, iter))
 	{
-	  char *newpath = g_strdup_printf ("%s/%s", path, name);
+	  gchar *newpath = g_strdup_printf ("%s/%s", path, name);
 	  count = gen_items (ui, newpath, &children, merge, count);
-	  free (newpath);
+	  g_free (newpath);
 	}
       }
     }
@@ -192,10 +193,11 @@ static void update_menu (void)
     gtk_ui_manager_insert_action_group (ui, media_group, 0);
     for (i = 0; (tree = menu_get_tree_name (i)) != NULL; ++i)
     {
-      char *key = g_strdup_printf ("/ui/%s/MediaMenu/MediaItems", tree);
-      gtk_tree_model_get_iter_first (mm_model, &iter);
-      gen_items (ui, key, &iter, media_merge, 0);
-      free (key);
+      gchar *key = g_strdup_printf ("/ui/%s/MediaMenu/MediaItems", tree);
+      if (gtk_tree_model_get_iter_first (mm_model, &iter)) {
+        gen_items (ui, key, &iter, media_merge, 0);
+      }
+      g_free (key);
     }
   }
 }
@@ -300,6 +302,7 @@ static void load_old_file (char *fname)
 	  g_printerr (_("mediamarks: media marks file %s corrupted - ignoring rest of file\n"),
 		   fname);
 	  free (id);
+          fclose(f);
 	  return;
 	}
 	parent = iter;
@@ -723,20 +726,21 @@ static void mm_response_cb (GtkDialog *dbox, int response, gpointer data)
 
 static gboolean map_cb (GtkWidget* widget, gpointer data)
 {
-  gdk_window_set_events (widget->window,
-			 gdk_window_get_events (widget->window) | GDK_BUTTON_PRESS_MASK);
+  GdkWindow *window = gtk_widget_get_window(widget);
+  gdk_window_set_events (window,
+                         gdk_window_get_events (window) | GDK_BUTTON_PRESS_MASK);
   return FALSE;
 }
 
 static const GtkActionEntry buttons_data[] = {
-  { "play",	GTK_STOCK_MEDIA_PLAY,	NULL,			NULL,		NULL,				play_cb },
-  { "new",	GTK_STOCK_NEW,		NULL,			NULL,		N_("Add an MRL"),		mm_add_show },
-  { "newcat",	GTK_STOCK_DIRECTORY,	N_("_Category"),	"<Control>F",	N_("Add a category folder"),	new_category_cb },
-  { "edit",	GTK_STOCK_EDIT,		NULL,			"<Control>E",	N_("Edit the selected item"),	edit_cb },
-  { "delete",	GTK_STOCK_DELETE,	NULL,			"Delete",	N_("Delete the selected item"),	delete_cb },
-  { "copy",	GTK_STOCK_COPY,		NULL,			NULL,		NULL,				copy_cb },
-  { "cut",	GTK_STOCK_CUT,		NULL,			NULL,		NULL,				cut_cb },
-  { "paste",	GTK_STOCK_PASTE,	NULL,			NULL,		NULL,				paste_cb },
+  { "play",    GTK_STOCK_MEDIA_PLAY,   NULL,                   NULL,           NULL,                           G_CALLBACK(play_cb) },
+  { "new",     GTK_STOCK_NEW,          NULL,                   NULL,           N_("Add an MRL"),               G_CALLBACK(mm_add_show) },
+  { "newcat",  GTK_STOCK_DIRECTORY,    N_("_Category"),        "<Control>F",   N_("Add a category folder"),    G_CALLBACK(new_category_cb) },
+  { "edit",    GTK_STOCK_EDIT,         NULL,                   "<Control>E",   N_("Edit the selected item"),   G_CALLBACK(edit_cb) },
+  { "delete",  GTK_STOCK_DELETE,       NULL,                   "Delete",       N_("Delete the selected item"), G_CALLBACK(delete_cb) },
+  { "copy",    GTK_STOCK_COPY,         NULL,                   NULL,           NULL,                           G_CALLBACK(copy_cb) },
+  { "cut",     GTK_STOCK_CUT,          NULL,                   NULL,           NULL,                           G_CALLBACK(cut_cb) },
+  { "paste",   GTK_STOCK_PASTE,        NULL,                   NULL,           NULL,                           G_CALLBACK(paste_cb) },
 };
 
 static const char buttons_structure[] =
@@ -801,7 +805,7 @@ void mediamarks_init (void)
 				NULL);
   gtk_window_set_default_size (GTK_WINDOW (manage_dlg), 400, 250);
   hide_on_delete (manage_dlg, &is_visible);
-  vbox = GTK_BOX (GTK_DIALOG (manage_dlg)->vbox);
+  vbox = GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (manage_dlg)));
 
   /* buttons & menus */
 
@@ -943,9 +947,10 @@ void mediamarks_init (void)
   g_signal_connect (G_OBJECT(cat_entry), "activate",
 		    G_CALLBACK(cat_close_cb), cat_dlg);
 */
-  gtk_box_pack_start_defaults (GTK_BOX (hbox), cat_entry);
+  gtk_box_pack_start (GTK_BOX (hbox), cat_entry, TRUE, TRUE, 0);
 
-  gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (cat_dlg)->vbox), hbox);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (cat_dlg))),
+                      hbox, TRUE, TRUE, 0);
 
   /* script engine functions */
 
