@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2007 the xine project
+ * Copyright (C) 2003-2017 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 
 #include <string.h>
 #include <stdlib.h>
-#include <gdk/gdkkeysyms.h>
 
 #include "menu.h"
 #include "playlist.h"
@@ -62,7 +61,7 @@ static void autoplay_cb (GtkAction *action, const char *data)
 {
   int n, i, pos;
 
-  char **mrls = xine_get_autoplay_mrls (xine, data, &n);
+  const char * const *mrls = xine_get_autoplay_mrls (xine, data, &n);
   if (!mrls)
   {
     playlist_logo (NULL);
@@ -109,7 +108,7 @@ static size_t add_autoplay_actions (GtkUIManager *ui)
 {
   const char *const *autoplay_ids = xine_get_autoplay_input_plugin_ids (xine);
   GtkActionGroup *actions;
-  char name[20];
+  char name[32];
   size_t i;
 
   static const struct {
@@ -159,7 +158,7 @@ static size_t add_vis_actions (GtkUIManager *ui, GtkAction *group_action)
     xine_list_post_plugins_typed (xine, XINE_POST_TYPE_AUDIO_VISUALIZATION);
   GtkActionGroup *actions;
   GSList *radiogroup;
-  char name[20];
+  char name[32];
   xine_cfg_entry_t entry;
   const char *vis;
   size_t i;
@@ -203,7 +202,7 @@ void menu_add_items (GtkUIManager *ui, const char *const prefix,
   int i;
   for (i = 0; i < 2; ++i)
   {
-    char *path = g_strdup_printf ("/ui/%s/%s", type[i], prefix);
+    gchar *path = g_strdup_printf ("/ui/%s/%s", type[i], prefix);
     size_t j = count;
     while (j--)
     {
@@ -211,7 +210,7 @@ void menu_add_items (GtkUIManager *ui, const char *const prefix,
       gtk_ui_manager_add_ui (ui, merge, path, name, name,
 			     GTK_UI_MANAGER_MENUITEM, TRUE);
     }
-    free (path);
+    g_free (path);
   }
 }
 
@@ -219,9 +218,9 @@ void menu_add_items (GtkUIManager *ui, const char *const prefix,
 #define JS_CB(func, command) \
   static void js_##func##_cb (GtkAction *action, gpointer data) \
   { \
-    char *fn = g_strdup_printf (_("menu function %s"), #func); \
+    gchar *fn = g_strdup_printf (_("menu function %s"), #func); \
     engine_exec ((command), NULL, NULL, fn); \
-    free (fn); \
+    g_free (fn); \
   }
 
 /* Radio action callback (no Javascript)
@@ -267,7 +266,7 @@ JS_CB (Quit, "exit ();")
 /* View */
 JS_CB_T (Fullscreen)
 {
-  if (!GTK_WIDGET_MAPPED (app))
+  if (!gtk_widget_get_mapped (app))
     app_show ();
   gtk_video_set_fullscreen ((GtkVideo *)gtv, v, &engine_lock);
   gtk_action_set_sensitive (action_items.hidemain, !v);
@@ -345,15 +344,17 @@ JS_CB (Pause, "if (!is_live_stream ()) pause ();")
 JS_CB (FFwd, "if (!is_live_stream ()) av_speed.v = 5;")
 JS_CB (Stop, "stop ();")
 
-#define CB(name, stock, label, key) \
-  { #name, (stock), (label), (key), NULL, G_CALLBACK(js_##name##_cb) }
+#define CB(aname, astock, alabel, akey) \
+  { .name = #aname, .stock_id = (astock), .label = (alabel), .accelerator = (akey), .callback = G_CALLBACK(js_##aname##_cb), }
+#define CM(aname, astock, alabel) \
+  { .name =  #aname, .stock_id = (astock), .label = (alabel) }
 static const GtkActionEntry main_actions[] = {
   /* Menu entry text */
-  { "FileMenu",		NULL,			N_("_File") },
+  CM(FileMenu,          NULL,                   N_("_File")),
   CB(Open,		GTK_STOCK_OPEN,		N_("_Open..."),			"<control>O"),
   CB(MRL,		GTK_STOCK_OPEN,		N_("Open _MRL..."),		"<control>M"),
   CB(Playlist,		GTK_STOCK_INDEX,	N_("Play_list..."),		NULL),
-  { "ConfigMenu",	GTK_STOCK_PREFERENCES,	N_("_Configure") },
+  CM(ConfigMenu,        GTK_STOCK_PREFERENCES,  N_("_Configure")),
   CB(Prefs,		NULL,			N_("_Preferences..."),		NULL),
   CB(Keys,		NULL,			N_("_Keybindings..."),		NULL),
   CB(Startup,		NULL,			N_("_Startup script..."),	NULL),
@@ -362,40 +363,40 @@ static const GtkActionEntry main_actions[] = {
 #endif
   CB(Snapshot,		NULL,			N_("_Snapshot..."),		"<control>S"),
   CB(Quit,		GTK_STOCK_QUIT,		N_("_Quit"),			"<control>Q"),
-  { "ViewMenu",		NULL,			N_("_View") },
+  CM(ViewMenu,          NULL,                   N_("_View")),
   CB(HideMain,		GTK_STOCK_CLOSE,	N_("_Hide video"),		"<control>W"),
   CB(ShowMain,		NULL,			N_("_Show video"),		NULL),
-  { "ToolbarMenu",	NULL,			N_("_Toolbars") },
-  { "WindowMenu",	NULL,			N_("_Window size") },
+  CM(ToolbarMenu,       NULL,                   N_("_Toolbars")),
+  CM(WindowMenu,        NULL,                   N_("_Window size")),
   CB(WindowRevert,	NULL,			N_("Re_set"),			NULL),
   CB(Keypad,		NULL,			N_("_Keypad..."),		NULL),
   CB(ZoomIn,		GTK_STOCK_ZOOM_IN,	N_("Zoom _in"),			NULL),
   CB(ZoomOut,		GTK_STOCK_ZOOM_OUT,	N_("Zoom _out"),		NULL),
   CB(Zoom100,		GTK_STOCK_ZOOM_100,	N_("_Zoom 100%"),		NULL),
-  { "AspectMenu",	NULL,			N_("_Aspect ratio"),		NULL },
-  { "VisMenu",		NULL,			N_("_Visualisations"),		NULL },
-  { "SubsMenu",		"gxine-settings-spu",	N_("_Subtitles"),		NULL },
-  { "ViewMisc",		GTK_STOCK_PROPERTIES,	N_("_Settings"),		NULL },
-  { "VideoMenu",	NULL,			N_("V_ideo"),			NULL },
+  CM(AspectMenu,        NULL,                   N_("_Aspect ratio")),
+  CM(VisMenu,           NULL,                   N_("_Visualisations")),
+  CM(SubsMenu,          "gxine-settings-spu",   N_("_Subtitles")),
+  CM(ViewMisc,          GTK_STOCK_PROPERTIES,   N_("_Settings")),
+  CM(VideoMenu,         NULL,                   N_("V_ideo")),
   CB(VideoSettings,	GTK_STOCK_PROPERTIES,	/* Video */ N_("_Controls..."),	NULL),
-  { "VideoConfigMenu",	GTK_STOCK_PREFERENCES,	N_("_Configure plugins"),	NULL },
+  CM(VideoConfigMenu,   GTK_STOCK_PREFERENCES,  N_("_Configure plugins")),
   CB(DeinterlaceConf,	NULL,			N_("_Deinterlace..."),		NULL),
   CB(VideoPPConf,	NULL,			N_("_Video post-processing..."),NULL),
-  { "AudioMenu",	NULL,			N_("_Audio"),			NULL },
+  CM(AudioMenu,         NULL,                   N_("_Audio")),
   CB(AudioSettings,	GTK_STOCK_PROPERTIES,	/* Audio */ N_("_Controls..."),	NULL),
   CB(AudioEq,		GTK_STOCK_PROPERTIES,	N_("_Equaliser..."),		NULL),
-  { "AudioConfigMenu",	GTK_STOCK_PREFERENCES,	N_("_Configure plugins"),		NULL },
+  CM(AudioConfigMenu,   GTK_STOCK_PREFERENCES,  N_("_Configure plugins")),
   CB(AudioPPConf,	NULL,			N_("_Audio post-processing..."),NULL),
-  { "MediaMenu",	NULL,			N_("_Media"),			NULL },
+  CM(MediaMenu,         NULL,                   N_("_Media")),
   CB(MediaAdd,		GTK_STOCK_NEW,		N_("_Add media mark..."),	"<control>D"),
   CB(MediaManage,	GXINE_MEDIA_MARK,	N_("_Manage media marks..."),	"<control>B"),
   CB(MediaImport,	GTK_STOCK_OPEN,		N_("_Import media marks..."),	NULL),
-  { "ToolsMenu",	NULL,			N_("_Tools"),			NULL },
+  CM(ToolsMenu,         NULL,                   N_("_Tools")),
   CB(EngineLog,		NULL,			N_("Engine _log..."),		"<control>L"),
   CB(JSConsole,		GTK_STOCK_EXECUTE,	N_("_Javascript console..."),	"<control>J"),
   CB(StreamInfo,	GTK_STOCK_INFO,		N_("Stream _info..."),		NULL),
   CB(SetupWizards,	GTK_STOCK_REDO,		N_("_Re-run setup wizards..."), NULL),
-  { "HelpMenu",		NULL,			N_("_Help"),			NULL },
+  CM(HelpMenu,          NULL,                   N_("_Help")),
   CB(About,		GTK_STOCK_ABOUT,	N_("_About..."),		NULL),
   CB(PrevTrack,	GTK_STOCK_MEDIA_PREVIOUS,	N_("Pre_vious track"),		NULL),
   CB(Play,	GTK_STOCK_MEDIA_PLAY,		N_("_Play"),			NULL),
@@ -405,9 +406,10 @@ static const GtkActionEntry main_actions[] = {
   CB(NextTrack,	GTK_STOCK_MEDIA_NEXT,		N_("_Next track"),		NULL),
 };
 #undef CB
+#undef CM
 
-#define CB(name, stock, label, key, state) \
-  { #name, (stock), (label), (key), NULL, G_CALLBACK(js_##name##_cb), (state) }
+#define CB(aname, astock, alabel, akey, astate) \
+  { .name = #aname, .stock_id = (astock), .label = (alabel), .accelerator = (akey), .callback = G_CALLBACK(js_##aname##_cb), .is_active = (astate), }
 typedef enum {
   TOGGLE_FULLSCREEN,
   TOGGLE_WINDOWEDMODE_TOOLBAR,
@@ -622,10 +624,10 @@ static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
   GError *error = NULL;
   char *key = NULL;
 
-  char *xml = g_strdup_printf ("<ui><%s name='%s'>%s</%s></ui>",
-			       type[i], type[i], def, type[i]);
+  gchar *xml = g_strdup_printf ("<ui><%s name='%s'>%s</%s></ui>",
+                                type[i], type[i], def, type[i]);
   gtk_ui_manager_add_ui_from_string (ui, xml, -1, &error);
-  free (xml);
+  g_free (xml);
 
   asreprintf (&key, "/ui/%s", type[i]);
   {
@@ -654,7 +656,7 @@ void scale_changed_cb (GtkWidget *widget, gdouble factor, gpointer data)
 {
   if (factor != -1 && action_items.resize_factor
       && xine_get_status (stream) != XINE_STATUS_IDLE
-      && GTK_WIDGET_VISIBLE (gtv))
+      && gtk_widget_get_visible (gtv))
   {
     logprintf ("gxine: received scale change event (%lf%%)\n", factor);
     GtkAction *action = find_video_size_action (factor);
@@ -667,9 +669,9 @@ void scale_changed_cb (GtkWidget *widget, gdouble factor, gpointer data)
       return;
     }
 
-    char *label = g_strdup_printf (_("_Other (%.1lf%%)"), factor);
+    gchar *label = g_strdup_printf (_("_Other (%.1lf%%)"), factor);
     g_object_set ((GObject *)action_items.resize_factor, "label", label, NULL);
-    free (label);
+    g_free (label);
     GSList *changed;
     foreach_glist (changed, gtk_radio_action_get_group
 			      (GTK_RADIO_ACTION (action_items.resize_factor)))
@@ -683,8 +685,9 @@ void scale_changed_cb (GtkWidget *widget, gdouble factor, gpointer data)
     if (changed)
       g_signal_handlers_unblock_by_func (changed->data, js_Window_cb, NULL);
   }
-  else
+  else {
     logprintf ("gxine: ignored scale change event (%lf%%)\n", factor);
+  }
 }
 
 static void update_subtitles_menu_internal (GtkActionGroup *action)
@@ -696,7 +699,7 @@ static void update_subtitles_menu_internal (GtkActionGroup *action)
   {
     static char subt[] = "Subtitles..";
     char lang[XINE_LANG_MAX];
-    sprintf (subt + 9, "%d", i);
+    snprintf (subt + 9, 3, "%d", i);
     GtkAction *item = get_action (subt);
     if (!item)
       break;
@@ -709,7 +712,7 @@ static void update_subtitles_menu_internal (GtkActionGroup *action)
       label = g_strconcat (label, strcmp (lang, "none") ? " / " : NULL, lang, NULL);
     }
     g_object_set ((GObject *) item, "label", label ? label : base, NULL);
-    free (label);
+    g_free (label);
   }
 }
 
