@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2007 the xine project
+ * Copyright (C) 2003-2017 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 #include "noskin_window.h"
 
 #include <stdio.h>
-#include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_XINERAMA
@@ -34,6 +33,8 @@
 #include <X11/Xatom.h>
 #endif
 
+#include <gtk/gtk.h>
+
 #include "ui.h"
 #include "utils.h"
 #include "engine.h"
@@ -53,11 +54,13 @@
 */
 #undef USE_NET_WM_STATE_SHADED
 
+#define GUI_DEFAULT_WIDTH  320
+#define GUI_DEFAULT_HEIGHT 240
+
 /* global */
 GtkWidget *gtv;
 
 /* private/local */
-static GtkTooltips    *tips;
 static int             have_video = -1, have_audio = -1;
 static struct app_prop_s {
   gboolean	       sticky, fs, real_sticky, shaded;
@@ -72,7 +75,7 @@ static Atom opacity_lock;
 static void gtk_window_set_shade (GtkWindow *w, gboolean shade)
 {
 #ifdef HAVE_X11
-  GdkWindow *window = ((GtkWidget *)w)->window;
+  GdkWindow *window = gtk_widget_get_window (GTK_WIDGET (w));
   Display *display = GDK_WINDOW_XDISPLAY (window);
   XEvent xev;
 
@@ -107,7 +110,7 @@ void window_check_vis (gboolean force)
     hv_new = 0;
   int ha_new = xine_get_stream_info (stream, XINE_STREAM_INFO_HAS_AUDIO);
   gboolean fs = gtk_video_is_fullscreen ((GtkVideo *)gtv);
-  gboolean was_shown = GTK_WIDGET_DRAWABLE (gtv);
+  gboolean was_shown = gtk_widget_is_drawable (gtv);
   gboolean is_shown = was_shown;
 
   if (force || have_video != hv_new || have_audio != ha_new)
@@ -212,9 +215,12 @@ static gint cw_height = 1;
 
 static gboolean window_fs_toolbar_move (GdkEventConfigure *event)
 {
+  GtkAllocation allocation = { 0 };
+  gtk_widget_get_allocation (app, &allocation);
+
   int h = fs_toolbar_at_top
 	  ? 1
-	  : (event ? event->height : app->allocation.height) - cw_height - 1;
+          : (event ? event->height : allocation.height) - cw_height - 1;
   int x, y;
   gtk_window_get_position (GTK_WINDOW(app), &x, &y);
   gtk_window_move (GTK_WINDOW(fs_toolbar),
@@ -248,8 +254,9 @@ static gboolean window_fs_toolbar_show_int (void)
   if (fs_is_sticky)
     gtk_window_stick (GTK_WINDOW(fs_toolbar));
 #ifdef HAVE_X11
-  XChangeProperty (GDK_WINDOW_XDISPLAY (fs_toolbar->window),
-		   GDK_WINDOW_XWINDOW (fs_toolbar->window), opacity_lock,
+  GdkWindow *window = gtk_widget_get_window(fs_toolbar);
+  XChangeProperty (GDK_WINDOW_XDISPLAY (window),
+                   GDK_WINDOW_XID (window), opacity_lock,
 		   XA_CARDINAL, 32, PropModeReplace,
 		   (unsigned char *)&have_video, 1);
 #endif
@@ -331,7 +338,7 @@ void window_wm_toolbar_reset (void)
 
 void window_wm_toolbar_restore (void)
 {
-  if (wm_toolbar && GTK_WIDGET_MAPPED (app))
+  if (wm_toolbar && gtk_widget_get_mapped (app))
     gtk_window_present (GTK_WINDOW (wm_toolbar));
 }
 
@@ -393,13 +400,13 @@ gboolean window_wm_toolbar_snap (void)
   if (!wm_toolbar)
     return FALSE;
 
-  gdk_window_get_frame_extents (app->window, &appframe);
-  gdk_window_get_frame_extents (wm_toolbar->window, &tbframe);
+  gdk_window_get_frame_extents (gtk_widget_get_window(app), &appframe);
+  gdk_window_get_frame_extents (gtk_widget_get_window(wm_toolbar), &tbframe);
 
   GdkRectangle monitor;
-  gdk_screen_get_monitor_geometry (((GtkWindow *)app)->screen,
-				   gdk_screen_get_monitor_at_window
-				     (((GtkWindow *)app)->screen, app->window),
+  GdkScreen *screen = gtk_window_get_screen(GTK_WINDOW(app));
+  gdk_screen_get_monitor_geometry (screen,
+                                   gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window(app)),
 				   &monitor);
 
   /* centre horizontally wrt video window */
@@ -430,7 +437,7 @@ gboolean window_wm_toolbar_snap (void)
   if (x != tbframe.x || y != tbframe.y)
   {
     if (can_lower)
-      gdk_window_lower (wm_toolbar->window);
+      gdk_window_lower (gtk_widget_get_window (wm_toolbar));
     gtk_window_move (GTK_WINDOW (wm_toolbar), x, y);
   }
 
@@ -439,7 +446,7 @@ gboolean window_wm_toolbar_snap (void)
 
 static gboolean popup_cb (GtkWidget *widget, gpointer data)
 {
-  if (GTK_WIDGET_VISIBLE (wm_toolbar ? : menubar))
+  if (gtk_widget_get_visible (wm_toolbar ? : menubar))
     gtk_menu_shell_select_first (GTK_MENU_SHELL (menubar), FALSE);
   else
     gtk_menu_popup (GTK_MENU (popup_menu), NULL, NULL, NULL, NULL, 0,
@@ -465,8 +472,8 @@ static void fs_show_hide (GtkWidget *widget, gpointer show)
 
 static gboolean refocus_cb (GtkWidget *widget)
 {
-  if (GTK_WIDGET_VISIBLE (widget) && !app_prop.shaded)
-    gdk_window_focus(widget->window, 0);
+  if (gtk_widget_get_visible (widget) && !app_prop.shaded)
+    gdk_window_focus (gtk_widget_get_window (widget), 0);
   return FALSE;
 }
 
@@ -505,7 +512,7 @@ app_state_cb (GtkWindow *window, GdkEventWindowState *event, gpointer data)
     if (!wm_toolbar)
     {
       gdk_flush ();
-      gtk_container_foreach (GTK_CONTAINER (window->bin.child),
+      gtk_container_foreach (GTK_CONTAINER (gtk_bin_get_child (GTK_BIN (window))),
 			     fs_show_hide, window);
       gdk_flush ();
     }
@@ -514,7 +521,7 @@ app_state_cb (GtkWindow *window, GdkEventWindowState *event, gpointer data)
 
     gtk_toggle_action_set_active (action_items.fullscreen, fs);
   }
-  else if (!GTK_WIDGET_VISIBLE (gtv))
+  else if (!gtk_widget_get_visible (gtv))
   {
     if (playlist_showing_logo ())
     {
@@ -530,12 +537,12 @@ app_state_cb (GtkWindow *window, GdkEventWindowState *event, gpointer data)
     if (fs_is_sticky)
       gtk_window_stick (window);
     if (!wm_toolbar)
-      gtk_container_foreach (GTK_CONTAINER (window->bin.child),
+      gtk_container_foreach (GTK_CONTAINER (gtk_bin_get_child (GTK_BIN (window))),
 			     fs_show_hide, NULL);
     else
       window_wm_toolbar_reset ();
     gtk_window_set_keep_above (window, TRUE);
-    gdk_window_focus(app->window, 0);
+    gdk_window_focus (gtk_widget_get_window (app), 0);
 
     gtk_toggle_action_set_active (action_items.fullscreen, fs);
   }
@@ -571,9 +578,10 @@ xevent_properties (void)
 {
   gdk_error_trap_push ();
 
-  Display *xd = GDK_WINDOW_XDISPLAY (app->window);
-  GdkDisplay *d = gdk_x11_lookup_xdisplay (xd);
-  Window xw = GDK_WINDOW_XID (app->window);
+  GdkWindow  *window = gtk_widget_get_window (app);
+  Display    *xd     = GDK_WINDOW_XDISPLAY (window);
+  GdkDisplay *d      = gdk_x11_lookup_xdisplay (xd);
+  Window xw = GDK_WINDOW_XID (window);
 
   Atom type;
   int fmt;
@@ -660,7 +668,7 @@ xevent_filter_cb (GdkXEvent *gx, GdkEvent *e, gpointer d)
   case ClientMessage:
     /* Lower the toolbar */
     if (wm_toolbar && wm_toolbar_snap && can_lower)
-      gdk_window_lower (wm_toolbar->window);
+      gdk_window_lower (gtk_widget_get_window (wm_toolbar));
     break;
 
   case FocusIn:
@@ -668,18 +676,22 @@ xevent_filter_cb (GdkXEvent *gx, GdkEvent *e, gpointer d)
 
   case FocusOut:
     if (gtk_video_is_fullscreen ((GtkVideo *) gtv)
-	&& GTK_WIDGET_MAPPED (gtv))
+        && gtk_widget_get_mapped (gtv))
     {
-      Display *xd = GDK_WINDOW_XDISPLAY (((GtkWidget *) d)->window);
+      Display *xd = GDK_WINDOW_XDISPLAY (gtk_widget_get_window (GTK_WIDGET (d)));
       Window fw = None;
       int fr;
       gdk_error_trap_push ();
       XGetInputFocus (xd, &fw, &fr);
+#if !GTK_CHECK_VERSION(2, 24, 0)
       GdkWindow *gw = gdk_window_lookup (fw);
-#ifdef HAVE_XINERAMA
-      if (gw ? gw == fs_toolbar->window : !XineramaIsActive (xd))
 #else
-      if (fs_toolbar->window && gw == fs_toolbar->window)
+      GdkWindow *gw = gdk_x11_window_lookup_for_display (gtk_widget_get_display (GTK_WIDGET (d)), fw);
+#endif
+#ifdef HAVE_XINERAMA
+      if (gw ? gw == gtk_widget_get_window(fs_toolbar) : !XineramaIsActive(xd))
+#else
+      if (gtk_widget_get_window(fs_toolbar) && gw == gtk_widget_get_window(fs_toolbar))
 #endif
 	XSetInputFocus (xd, x->xany.window, RevertToParent, CurrentTime);
       gdk_error_trap_pop ();
@@ -699,7 +711,7 @@ xevent_filter_cb (GdkXEvent *gx, GdkEvent *e, gpointer d)
     break;
 
   case PropertyNotify:
-    if (app->window)
+    if (gtk_widget_get_window (app))
       xevent_properties ();
     break;
   }
@@ -708,7 +720,7 @@ xevent_filter_cb (GdkXEvent *gx, GdkEvent *e, gpointer d)
 }
 #endif
 
-static gboolean noskin_main_init_cb (gpointer fullscreen)
+void noskin_main_post_init (gint fullscreen)
 {
   gtk_widget_add_events (app, GDK_KEY_RELEASE_MASK);
   g_object_connect (G_OBJECT (app),
@@ -725,8 +737,9 @@ static gboolean noskin_main_init_cb (gpointer fullscreen)
   ui_set_status (xine_get_param (stream, XINE_PARAM_AUDIO_MUTE)
 		 ? UI_AUDIO_MUTE : UI_AUDIO_UNMUTE);
 #ifdef HAVE_X11
-  gdk_window_set_events (app->window, GDK_ALL_EVENTS_MASK); /* seems to work */
-  gdk_window_add_filter (app->window, xevent_filter_cb, app);
+  GdkWindow *window = gtk_widget_get_window(app);
+  gdk_window_set_events (window, GDK_ALL_EVENTS_MASK); /* seems to work */
+  gdk_window_add_filter (window, xevent_filter_cb, app);
 #endif
   gtk_widget_grab_focus (app);
 
@@ -740,8 +753,6 @@ static gboolean noskin_main_init_cb (gpointer fullscreen)
 		  GDK_WINDOW_XWINDOW (app->window),
 		  RevertToParent, CurrentTime);
 */
-  gxine_init_done ();
-  return FALSE;
 }
 
 /* initialisation */
@@ -753,11 +764,13 @@ static gboolean app_configure_cb (GtkWidget *widget, GdkEventConfigure *event,
   if (gtk_video_is_fullscreen ((GtkVideo *)gtv))
   {
 #ifdef HAVE_XRANDR
+    GtkAllocation allocation = { 0 };
+    gtk_widget_get_allocation (fs_toolbar, &allocation);
     if (fs_toolbar_visible)
       window_fs_toolbar_move (event);
     window_fs_toolbar_set_geometry ();
     gtk_window_resize (GTK_WINDOW (fs_toolbar), event->width,
-		       fs_toolbar->allocation.height);
+                       allocation.height);
 #endif
   }
   else if (wm_toolbar_snap)
@@ -795,7 +808,7 @@ wm_show_hide_setup (gpointer object)
 		    NULL);
 }
 
-static gboolean noskin_post_init (gboolean unused)
+void noskin_post_init (void)
 {
   xine_cfg_entry_t entry;
   if (xine_config_lookup_entry (xine, "gui.window_size", &entry))
@@ -805,20 +818,15 @@ static gboolean noskin_post_init (gboolean unused)
   }
   if (xine_config_lookup_entry (xine, "gui.windowed_mode.magnify_lowres_video", &entry))
     gtk_video_set_auto_rescale ((GtkVideo *)gtv, entry.num_value);
-
-  gxine_init_done ();
-  return FALSE;
 }
 
 
-void noskin_main_init (const gchar *geometry, gboolean fullscreen)
+void noskin_main_init (const char *video_driver_id, const gchar *geometry, gboolean fullscreen)
 {
   GtkWidget     *vbox, *w;
   GtkWindow     *W;
   xine_cfg_entry_t entry;
 
-  tips = gtk_tooltips_new ();
-
   /* create app windows */
 
   app = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -875,21 +883,19 @@ void noskin_main_init (const gchar *geometry, gboolean fullscreen)
    * video window
    */
 
-  gtk_window_set_default_size (GTK_WINDOW (app), 320, 240);
+  gtk_window_set_default_size (GTK_WINDOW (app), GUI_DEFAULT_WIDTH, GUI_DEFAULT_HEIGHT);
 
   /*
    * video widget
    */
 
   gtv = gtk_video_new (xine, stream, xine_get_video_source (stream),
-		       video_driver_id, 320, 240,
+		       video_driver_id, GUI_DEFAULT_WIDTH, GUI_DEFAULT_HEIGHT,
 		       0x04 /* press: button 2 */,
 		       0x08 /* release: button 3 */);
   if (xine_config_lookup_entry (xine, "gui.windowed_mode.auto_resize", &entry))
     gtk_video_set_auto_resize ((GtkVideo *)gtv, entry.num_value);
 
-  gxine_init_add ((GSourceFunc) noskin_post_init, NULL);
-
   if (!wm_toolbar)
   {
     /*
@@ -992,14 +998,15 @@ void noskin_main_init (const gchar *geometry, gboolean fullscreen)
     gtk_window_parse_geometry (GTK_WINDOW (app), geometry);
   gtk_widget_show (app);
 
-  gdk_window_set_decorations (app->window, GDK_DECOR_ALL | GDK_DECOR_MAXIMIZE);
-  gdk_window_set_functions (app->window, GDK_FUNC_ALL | GDK_FUNC_MAXIMIZE);
+  GdkWindow *window = gtk_widget_get_window(app);
+  gdk_window_set_decorations (window, GDK_DECOR_ALL | GDK_DECOR_MAXIMIZE);
+  gdk_window_set_functions (window, GDK_FUNC_ALL | GDK_FUNC_MAXIMIZE);
 
   if (wm_toolbar)
   {
     gtk_widget_show_all (wm_toolbar);
 #ifdef HAVE_X11
-    XSelectInput (GDK_WINDOW_XDISPLAY (app->window), GDK_WINDOW_XID (app->window),
+    XSelectInput (GDK_WINDOW_XDISPLAY (window), GDK_WINDOW_XID (window),
 		  VisibilityChangeMask);
 #endif
     if (fullscreen)
@@ -1024,13 +1031,12 @@ void noskin_main_init (const gchar *geometry, gboolean fullscreen)
   have_video = 1;
 
 #ifdef HAVE_X11
-  Display *d = GDK_WINDOW_XDISPLAY (app->window);
+  Display *d = GDK_WINDOW_XDISPLAY (window);
   opacity_lock = XInternAtom (d, "_NET_WM_WINDOW_OPACITY_LOCKED", False);
-  XChangeProperty (d, GDK_WINDOW_XWINDOW (app->window), opacity_lock,
+  XChangeProperty (d, GDK_WINDOW_XID (window), opacity_lock,
 		   XA_CARDINAL, 32, PropModeReplace,
 		   (unsigned char *)&have_video, 1);
 #endif
 
-  gxine_init_add ((GSourceFunc) noskin_main_init_cb, GINT_TO_POINTER (fullscreen));
   g_timeout_add (500, update_slider_cb, NULL);
 }
