@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2007 the xine project
+ * Copyright (C) 2003-2017 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,9 +23,12 @@
 
 #include <glib.h>
 
-void noskin_main_init (const gchar *geometry, gboolean fullscreen);
+void noskin_main_init (const char *video_driver_id, const gchar *geometry, gboolean fullscreen);
 void noskin_main_close (void);
 
+void noskin_post_init (void);
+void noskin_main_post_init (gint fullscreen);
+
 void app_show (void);
 void app_hide (void);
 
