@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007 the xine-project
+ * Copyright (C) 2004-2017 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,16 +21,18 @@
 
 #include "globals.h"
 
-#include <pthread.h>
-#include <xine.h>
 #include <string.h>
 #include <stdlib.h>
-#include <gtk/gtkradiomenuitem.h>
+#include <pthread.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include <gtk/gtk.h>
+
+#include <xine.h>
+
 #include "globals.h"
 #include "menu.h"
 #include "ui.h"
@@ -137,19 +139,30 @@ static void play_exec_error (void)
     g_idle_add ((GSourceFunc)playlist_logo, NULL);
 }
 
+static void my_gdk_threads_leave(void *p)
+{
+  gdk_threads_leave();
+}
+
+static void my_pthread_mutex_unlock(void *p)
+{
+  pthread_mutex_unlock(p);
+}
+
 static __attribute__ ((noreturn)) void *play_exec (void *data)
 {
   xine_cfg_entry_t entry;
-  char *hash, *mrl = NULL;
+  const char *hash;
+  gchar *mrl = NULL;
 
   pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, NULL);
   pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 
   /*pthread_mutex_lock (&engine_lock);*/
   pthread_mutex_lock (&widgets_update_lock);
-  pthread_cleanup_push (pthread_mutex_unlock, &widgets_update_lock);
+  pthread_cleanup_push (my_pthread_mutex_unlock, &widgets_update_lock);
   gdk_threads_enter();
-  pthread_cleanup_push (gdk_threads_leave, NULL);
+  pthread_cleanup_push (my_gdk_threads_leave, NULL);
 
   gxineinfo_clear (infobars);
   gxineinfo_update_line (infobars, 0, "%s", _("opening..."));
@@ -206,9 +219,9 @@ static __attribute__ ((noreturn)) void *play_exec (void *data)
 
       if (i < G_N_ELEMENTS (subs))
       {
-	char *tmp = g_filename_to_uri (file, NULL, NULL);
+        gchar *tmp = g_filename_to_uri (file, NULL, NULL);
 	mrl = g_strconcat (cur_mrl, hash ? ";subtitle:" : "#subtitle:", tmp + 7, NULL);
-	free (tmp);
+        g_free (tmp);
       }
     }
   }
@@ -219,7 +232,7 @@ static __attribute__ ((noreturn)) void *play_exec (void *data)
   if (ret)
   {
     gdk_threads_enter ();
-    pthread_cleanup_push (gdk_threads_leave, NULL);
+    pthread_cleanup_push (my_gdk_threads_leave, NULL);
     ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
     infobar_show_metadata (infobars);
     pthread_cleanup_pop (TRUE);
@@ -233,7 +246,7 @@ static __attribute__ ((noreturn)) void *play_exec (void *data)
 
     /* correct for possible stream changes */
     gdk_threads_enter ();
-    pthread_cleanup_push (gdk_threads_leave, NULL);
+    pthread_cleanup_push (my_gdk_threads_leave, NULL);
     ui_set_status (playlist_showing_logo () ? UI_STOP : UI_PLAY);
     infobar_show_metadata (infobars);
     playlist_check_set_title ();
@@ -242,12 +255,12 @@ static __attribute__ ((noreturn)) void *play_exec (void *data)
   else
   {
     gdk_threads_enter ();
-    pthread_cleanup_push (gdk_threads_leave, NULL);
+    pthread_cleanup_push (my_gdk_threads_leave, NULL);
     play_exec_error ();
     pthread_cleanup_pop (TRUE);
   }
 
-  free (mrl);
+  g_free (mrl);
   pthread_cleanup_pop (TRUE);
   /*pthread_mutex_unlock (&engine_lock);*/
   play_thread_created = 0;
@@ -339,7 +352,7 @@ static int listen_speed (void *se, se_t *se_t, se_o_t *o, se_prop_t *prop,
   return FALSE;
 }
 
-static void get_speed (se_prop_t *prop, se_prop_read_t *value)
+static void get_speed (const struct ui_property_s *prop, se_prop_read_t *value)
 {
   int speed = xine_get_param (stream, XINE_PARAM_SPEED);
   int bit = 0;
@@ -402,9 +415,10 @@ static void set_aspect (int aspect)
 static void update_postproc_chain (const char *pref, const char *value)
 {
   xine_cfg_entry_t entry;
-  xine_config_lookup_entry (xine, pref, &entry);
-  entry.str_value = (char *)value;
-  preferences_update_entry (&entry);
+  if (xine_config_lookup_entry (xine, pref, &entry)) {
+    entry.str_value = (char *)value;
+    preferences_update_entry (&entry);
+  }
 }
 
 static JSBool set_postproc_chain (const char *func, const char *pref,
@@ -895,7 +909,7 @@ static JSBool js_snapshot (JSContext *cx, uintN argc, jsval *vp)
   jsval    *argv = JS_ARGV (cx, vp);
 
   se_log_fncall_checkinit ("snapshot");
-  se_argc_check_range (0, 3, "snapshot");
+  se_argc_check_max (3, "snapshot");
 
   switch (argc) /* note fall-through */
   {
@@ -1006,8 +1020,9 @@ static JSBool controls_play (JSContext *cx, uintN argc, jsval *vp)
 	break;
       }
       play_item_dispose (item);
-    } else
+    } else {
       logprintf (_("script_engine: error: no valid play item available\n"));
+    }
   }
   set_speed (XINE_SPEED_NORMAL);
 
@@ -1071,14 +1086,14 @@ const char *player_get_cur_mrl (void)
 }
 
 
-static xine_audio_port_t *load_audio_out_driver (void)
+static xine_audio_port_t *load_audio_out_driver (const char *audio_driver_id)
 {
   xine_audio_port_t      *audio_port;
 
-  char **choices = get_driver_ids (xine_list_audio_output_plugins (xine));
+  const char * const * choices = xine_list_audio_output_plugins (xine);
   /* try to init audio with stored information */
   int i = xine_config_register_enum (xine,
-				     "audio.driver", 0, choices,
+                                     "audio.driver", 0, (char **)choices,
 				     _("audio driver to use"),
 				     NULL, 10, GXINE_TRUE, NULL);
   if (!audio_driver_id)
@@ -1118,7 +1133,7 @@ static void add_chain_funcs (const char *prop, const char *config,
 			listen_postproc_chain, (void *)config);
 }
 
-void player_init (void)
+void player_init (const char *audio_driver_id)
 {
 /*
   se_o_t            *controls_obj;
@@ -1128,7 +1143,7 @@ void player_init (void)
 
   /* load drivers, create xine stream */
 
-  audio_port = load_audio_out_driver ();
+  audio_port = load_audio_out_driver (audio_driver_id);
   if (!audio_port)
   {
     display_error_modal (FROM_GXINE, _("Fatal error"),
@@ -1284,12 +1299,12 @@ void player_init (void)
   }
 
   {
-    static char *window_size_labels[] = {
+    static const char *const window_size_labels[] = {
       N_("50%"), N_("75%"), N_("100%"), N_("150%"), N_("200%"),
       NULL
     };
     xine_config_register_enum
-      (xine, "gui.window_size", 0, window_size_labels,
+      (xine, "gui.window_size", 0, (char **)window_size_labels,
        _("The main window's default size"),
        NULL, 0, NULL, NULL);
     xine_config_register_bool
