@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2007 the xine-project
+ * Copyright (C) 2002-2017 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,10 +34,10 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkkeysyms.h>
 #include <glib.h>
+#include <gdk/gdk.h>
+#include <gtk/gtk.h>
+
 #include <xine/xmlparser.h>
 
 #include <pthread.h>
@@ -99,7 +99,7 @@ static int playlist_clip (int);
 
 static play_item_t *peek_play_item (GtkTreeIter *iter)
 {
-  GValue v = {};
+  GValue v = G_VALUE_INIT;
   gtk_tree_model_get_value (pl_model, iter, COLUMN_PLAY_ITEM, &v);
   void *p = g_value_peek_pointer (&v);
   g_value_unset (&v);
@@ -126,7 +126,7 @@ static inline void cur_item_dispose (void)
 
 static gboolean item_marked_current (GtkTreeIter *iter)
 {
-  GValue v = {};
+  GValue v = G_VALUE_INIT;
   gtk_tree_model_get_value (pl_model, iter, COLUMN_MARKING, &v);
   const char *p = g_value_peek_pointer (&v);
   gboolean ret = p && *p;
@@ -184,8 +184,9 @@ static gboolean playlist_browse_set_cb (const play_item_t *item)
 
 static gboolean map_cb (GtkWidget* widget, gpointer data)
 {
-  gdk_window_set_events (widget->window,
-			 gdk_window_get_events (widget->window) | GDK_BUTTON_PRESS_MASK);
+  GdkWindow *window = gtk_widget_get_window (widget);
+  gdk_window_set_events (window,
+                         gdk_window_get_events (window) | GDK_BUTTON_PRESS_MASK);
   return FALSE;
 }
 
@@ -527,6 +528,9 @@ static int playlist_load (const char *fname)
     goto ret0;
   }
 
+  xml_parser_free_tree (root);
+  xml_parser_finalize_R (xml);
+
   ret = 1;
   return 1;
 
@@ -690,7 +694,7 @@ static inline void play_item_check_autoplay (play_item_t *item)
   if (autoplay_ids)
     for (i = 0; autoplay_ids[i]; ++i)
     {
-      char **mrls = xine_get_autoplay_mrls (xine, autoplay_ids[i], &j);
+      const char * const *mrls = xine_get_autoplay_mrls (xine, autoplay_ids[i], &j);
       if (mrls)
 	for (j = 0; mrls[j]; ++j)
 	  if (!strcmp (mrls[j], item->mrl))
@@ -1089,7 +1093,7 @@ pl_m3u_fetch (const char *const *lines, size_t index, void *data, const char *mr
     tmp = g_strconcat (lines[index], suffix, NULL);
 
   play_item_t *item = play_item_new (NULL, tmp, 0, 0);
-  free (tmp);
+  g_free (tmp);
 
   return item;
 }
@@ -1144,6 +1148,8 @@ pl_pls_parse (const char *const *lines, void **data)
 static play_item_t *
 pl_pls_fetch (const char *const *lines, size_t index, void *data, const char *mrl)
 {
+  play_item_t *ret = NULL;
+
   if (++index > *(unsigned int *)data)
     return NULL;
 
@@ -1168,13 +1174,12 @@ pl_pls_fetch (const char *const *lines, size_t index, void *data, const char *mr
 	title[pos] = 0;
     }
 
-    play_item_t *ret = play_item_new (title, file, 0, 0);
+    ret = play_item_new (title, file, 0, 0);
     g_free (file);
-
-    return ret;
   }
 
-  return NULL;
+  g_free (title);
+  return ret;
 }
 
 static int
@@ -1303,12 +1308,12 @@ static int playlist_load_asx (const char *mrl, gint ins_pos)
 		g_printerr (_("playlist: no HREF property\n"));
 
 	    }
-	    else if (strcmp(current_node_l2->name, "TITLE") == 0)
+            else if (strcmp(current_node_l2->name, "TITLE") == 0) {
 	      title = current_node_l2->data;
-	    else
+            } else {
 	      logprintf ("playlist: unknown tag %s detected\n",
 			 current_node_l2->name);
-
+            }
 	    if (current_node_l2)
 	      current_node_l2 = current_node_l2->next;
 	  } /* end while */
@@ -1689,7 +1694,7 @@ static char *auth_escape (GtkEntry *entry)
     unsigned char c = (unsigned char) *text++;
     if (c < 33 || c == '/' || c == ':' || c == '@' || c > '~')
     {
-      sprintf (ptr, "%%%02X", c);
+      snprintf (ptr, 4, "%%%02X", c);
       ptr += 3;
     }
     else
@@ -1701,7 +1706,7 @@ static char *auth_escape (GtkEntry *entry)
 
 static int display_auth (int args, const char *mrl)
 {
-  if (GTK_WIDGET_VISIBLE ((GtkWidget *) auth.dbox))
+  if (gtk_widget_get_visible (GTK_WIDGET(auth.dbox)))
     return 0;
 
   const char *host = strstr (mrl, "://");
@@ -1710,9 +1715,9 @@ static int display_auth (int args, const char *mrl)
 
   int retval = 2;
 
-  char *txt = g_strdup_printf (_("Authorisation required for %s"), mrl);
+  gchar *txt = g_strdup_printf (_("Authorisation required for %s"), mrl);
   gtk_window_set_title (GTK_WINDOW (auth.dbox), txt);
-  free (txt);
+  g_free (txt);
   gtk_label_set_text (auth.label, args > 1 ? mrl + strlen (mrl) + 1 : "");
   gtk_entry_set_text (auth.name, "");
   gtk_entry_set_text (auth.pass, "");
@@ -1743,15 +1748,15 @@ static int display_auth (int args, const char *mrl)
     free (name);
     free (pass);
 
-    play_item_t *play_item;
+    play_item_t *play_item = NULL;
     int pos;
     for (pos = 0; pos < playlist_size (); ++pos)
       if (!strcmp ((play_item = playlist_get_item (pos))->mrl, mrl))
 	break;
-    if (pos == playlist_size ())
+    if (!play_item || pos == playlist_size ())
     {
       pos = playlist_add (play_item_new (NULL, txt, 0, 0), -1);
-      free (txt);
+      g_free (txt);
     }
     else
     {
@@ -1776,7 +1781,6 @@ static int display_auth (int args, const char *mrl)
       }
   }
 
-ret:
   gtk_widget_hide ((GtkWidget *) auth.dbox);
   gtk_entry_set_text (auth.name, "");
   gtk_entry_set_text (auth.pass, "");
@@ -1790,7 +1794,6 @@ static void xine_event_cb (void *user_data, const xine_event_t *event)
   /*
   g_print ("playlist: got an event %d\n", event->type);
   */
-  static gboolean mrl_ext = FALSE;
 
   switch (event->type)
   {
@@ -1823,11 +1826,11 @@ static void xine_event_cb (void *user_data, const xine_event_t *event)
       else
       {
 	struct timeval  tv;
-	int             age;
+        long            age;
 
 	/* filter out old events */
 	gettimeofday (&tv, NULL);
-	age = abs (tv.tv_sec - event->tv.tv_sec);
+        age = labs (tv.tv_sec - event->tv.tv_sec);
 	if (age > 0)
 	{
 	  gdk_threads_leave();
@@ -1855,25 +1858,15 @@ static void xine_event_cb (void *user_data, const xine_event_t *event)
   case XINE_EVENT_SPU_BUTTON:
     {
       xine_spu_button_t *btn = event->data;
-      gtk_video_in_spu_button (gtv, btn->direction);
+      gtk_video_in_spu_button ((GtkVideo *)gtv, btn->direction);
     }
     break;
 
-#ifndef XINE_EVENT_MRL_REFERENCE_EXT
-/* present in 1.1.0 but not 1.0.2 */
-#define XINE_EVENT_MRL_REFERENCE_EXT 13
-typedef struct {
-  int alternative, start_time, duration;
-  const char mrl[1];
-} xine_mrl_reference_data_ext_t;
-#endif
   case XINE_EVENT_MRL_REFERENCE_EXT:
     {
       xine_mrl_reference_data_ext_t *ref = event->data;
       const char *title = ref->mrl + strlen (ref->mrl) + 1;
 
-      mrl_ext = TRUE;
-
       logprintf ("playlist: got a reference event (ref MRL: %s) (title: %s)\n",
 		 ref->mrl, title);
 
@@ -1904,39 +1897,6 @@ typedef struct {
       }
     }
     break;
-  case XINE_EVENT_MRL_REFERENCE:
-    if (!mrl_ext)
-    {
-      xine_mrl_reference_data_t *ref = (xine_mrl_reference_data_t *) event->data;
-
-      logprintf ("playlist: got a reference event (ref MRL: %s)\n", ref->mrl);
-
-      if (ref->alternative == 0)
-      {
-	play_item_t *item = play_item_new (NULL, ref->mrl, 0, 0);
-	item->type = PLAY_ITEM_PLAYLIST;
-	gdk_threads_enter ();
-
-	if (ref_list_pos == -1)
-	{
-	  playlist_flush (PLAY_ITEM_PLAYLIST);
-	  ref_list_pos = cur_list_pos + 1;
-	  playlist_add (item, ref_list_pos);
-	  /* remove the item which triggered this reference event */
-	  /*
-	  GtkTreeIter iter;
-	  gtk_tree_model_iter_nth_child (pl_model, &iter,
-					 NULL, cur_list_pos--);
-	  gtk_list_store_remove (pl_store, &iter);
-	  */
-	}
-	else
-	  playlist_add (item, ++ref_list_pos);
-
-	gdk_threads_leave ();
-      }
-    }
-    break;
 
   case XINE_EVENT_UI_MESSAGE:
     {
@@ -1950,6 +1910,7 @@ typedef struct {
       {
       default:
 	g_print (_("xine_event_cb: unknown XINE_MSG_*\n"));
+        /* fall thru */
       case XINE_MSG_NO_ERROR:
       case XINE_MSG_AUDIO_OUT_UNAVAILABLE:
 	msg = display_info;
@@ -2011,7 +1972,7 @@ typedef struct {
       {
 	int i;
 	const char *param = (char *)data + data->parameters;
-	char *txt = strdup ("");
+        gchar *txt = g_strdup ("");
 	for (i = 0; i < data->num_parameters; ++i)
 	{
 	  asreprintf (&txt, "%s%s%s", txt, " " + !i, param);
@@ -2021,7 +1982,7 @@ typedef struct {
 	     data->explanation ? (char *)data + data->explanation
 			       : fallback ? : NULL,
 	     "%s", txt);
-	free (txt);
+        g_free (txt);
       }
       else if (data->explanation)
 	msg (FROM_XINE, (char *)data + data->explanation, "");
@@ -2094,7 +2055,7 @@ static void drop_cb (GtkTreeView	*widget,
   logprintf ("drag_drop: drop callback, length=%d, format=%d, pos=%d,%d, insert=%d\n",
 	     data->length, data->format, x, y, ins_pos);
 
-  if (data->format == 8)
+  if (gtk_selection_data_get_format (data) == 8)
   {
     /* Drag is from an external source */
     dnd_add_mrls (data, &cur_list_pos, ins_pos);
@@ -2354,20 +2315,20 @@ void playlist_play_from (int list_pos, int pos, int pos_time)
 }
 
 static const GtkActionEntry buttons_data[] = {
-  { "open",	GTK_STOCK_OPEN,		NULL,			NULL,		N_("Open a playlist file"),			open_cb },
-  { "saveas",	GTK_STOCK_SAVE_AS,	NULL,			NULL,		N_("Save the playlist file"),			save_as_cb },
-  { "clear",	GTK_STOCK_CLEAR,	NULL,			NULL,		N_("Clear the playlist"),			clear_cb },
-  { "fromfile",	GTK_STOCK_OPEN,		N_("_Add files"),	NULL,		N_("Add files to the playlist"),		add_cb },
-  { "new",	GTK_STOCK_NEW,		NULL,			NULL,		N_("New playlist item"),			new_cb },
-  { "edit",	GTK_STOCK_EDIT,		NULL,			"<Control>E",	N_("Edit the selected item"),			edit_cb },
-  { "delete",	GTK_STOCK_DELETE,	NULL,			"Delete",	N_("Delete the selected item"),			del_cb },
-  { "copy",	GTK_STOCK_COPY,		NULL,			NULL,		NULL,						copy_cb },
-  { "cut",	GTK_STOCK_CUT,		NULL,			NULL,		NULL,						cut_cb },
-  { "paste",	GTK_STOCK_PASTE,	NULL,			NULL,		NULL,						paste_cb },
-  { "play",	GTK_STOCK_MEDIA_PLAY,	NULL,			NULL,		NULL,						play_cb },
-  { "mm",	GXINE_MEDIA_MARK,	N_("Add to _media marks"), "<Control>M",N_("Add the selected item to the media marks list"), make_mediamark_cb },
-  { "prev",	GTK_STOCK_MEDIA_PREVIOUS, NULL,			NULL,		N_("Play previous item"),			playlist_prev_cb },
-  { "next",	GTK_STOCK_MEDIA_NEXT,	NULL,			NULL,		N_("Play next item"),				playlist_next_cb },
+  { "open",     GTK_STOCK_OPEN,        NULL,                   NULL,           N_("Open a playlist file"),                     G_CALLBACK(open_cb) },
+  { "saveas",   GTK_STOCK_SAVE_AS,     NULL,                   NULL,           N_("Save the playlist file"),                   G_CALLBACK(save_as_cb) },
+  { "clear",    GTK_STOCK_CLEAR,       NULL,                   NULL,           N_("Clear the playlist"),                       G_CALLBACK(clear_cb) },
+  { "fromfile", GTK_STOCK_OPEN,        N_("_Add files"),       NULL,           N_("Add files to the playlist"),                G_CALLBACK(add_cb) },
+  { "new",      GTK_STOCK_NEW,         NULL,                   NULL,           N_("New playlist item"),                        G_CALLBACK(new_cb) },
+  { "edit",     GTK_STOCK_EDIT,        NULL,                   "<Control>E",   N_("Edit the selected item"),                   G_CALLBACK(edit_cb) },
+  { "delete",   GTK_STOCK_DELETE,      NULL,                   "Delete",       N_("Delete the selected item"),                 G_CALLBACK(del_cb) },
+  { "copy",     GTK_STOCK_COPY,        NULL,                   NULL,           NULL,                                           G_CALLBACK(copy_cb) },
+  { "cut",      GTK_STOCK_CUT,         NULL,                   NULL,           NULL,                                           G_CALLBACK(cut_cb) },
+  { "paste",    GTK_STOCK_PASTE,       NULL,                   NULL,           NULL,                                           G_CALLBACK(paste_cb) },
+  { "play",     GTK_STOCK_MEDIA_PLAY,  NULL,                   NULL,           NULL,                                           G_CALLBACK(play_cb) },
+  { "mm",       GXINE_MEDIA_MARK,      N_("Add to _media marks"), "<Control>M",N_("Add the selected item to the media marks list"), G_CALLBACK(make_mediamark_cb) },
+  { "prev",     GTK_STOCK_MEDIA_PREVIOUS, NULL,                NULL,           N_("Play previous item"),                       G_CALLBACK(playlist_prev_cb) },
+  { "next",     GTK_STOCK_MEDIA_NEXT,  NULL,                   NULL,           N_("Play next item"),                           G_CALLBACK(playlist_next_cb) },
 };
 
 static const char buttons_structure[] =
@@ -2501,7 +2462,7 @@ void playlist_init (void)
 	"signal::response", G_CALLBACK(close_cb), NULL,
 	NULL);
 
-  vbox = GTK_BOX (GTK_DIALOG (dlg)->vbox);
+  vbox = GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg)));
 
   gtk_box_pack_start (vbox, gtk_ui_manager_get_widget (pl_ui, "/ui/toolbar"),
 		      FALSE, FALSE, 0);
@@ -2589,7 +2550,7 @@ void playlist_init (void)
   random_button = (GtkToggleButton *)gtk_check_button_new_with_mnemonic (_("_Random"));
   gtk_box_pack_start (GTK_BOX (hbox), (GtkWidget *)random_button, FALSE, FALSE, 2);
 
-  sprintf (logo_mrl, "%s/logo."LOGO_FORMAT, logodir);
+  snprintf (logo_mrl, sizeof(logo_mrl), "%s/logo."LOGO_FORMAT, logodir);
   logo_mode = 1;
 
   playlist_load (NULL);
@@ -2601,14 +2562,20 @@ void playlist_init (void)
 
   auth.dbox = GTK_DIALOG (gtk_dialog_new_with_buttons
 			    ("", (GtkWindow *) app,
+#if !GTK_CHECK_VERSION(3, 0, 0)
 			     GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
+#else
+                             GTK_DIALOG_MODAL,
+#endif
 			     GTK_STOCK_CANCEL, GTK_RESPONSE_DELETE_EVENT,
 			     GTK_STOCK_OPEN, GTK_RESPONSE_OK,
 			     NULL));
   gtk_dialog_set_default_response (auth.dbox, GTK_RESPONSE_OK);
   auth.label = GTK_LABEL (gtk_label_new (""));
-  auth.name = GTK_ENTRY (gtk_entry_new_with_max_length (64));
-  auth.pass = GTK_ENTRY (gtk_entry_new_with_max_length (128));
+  auth.name = GTK_ENTRY (gtk_entry_new ());
+  auth.pass = GTK_ENTRY (gtk_entry_new ());
+  gtk_entry_set_max_length (auth.name, 64);
+  gtk_entry_set_max_length (auth.pass, 128);
   gtk_entry_set_visibility (GTK_ENTRY (auth.pass), FALSE);
   GtkWidget *table = gtk_table_new (2, 2, FALSE);
   add_table_row_items (table, 0, 1, _("User name"), auth.name, NULL);
@@ -2619,11 +2586,13 @@ void playlist_init (void)
 						GTK_ICON_SIZE_DIALOG),
 		      FALSE, FALSE, 0);
   gtk_box_pack_start (vbox, table, TRUE, TRUE, 0);
-  gtk_box_pack_start ((GtkBox *) auth.dbox->vbox, (GtkWidget *) auth.label,
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (auth.dbox)),
+                      GTK_WIDGET (auth.label),
 		      TRUE, TRUE, 2);
-  gtk_box_pack_start ((GtkBox *) auth.dbox->vbox, (GtkWidget *) vbox,
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (auth.dbox)),
+                      GTK_WIDGET (vbox),
 		      TRUE, TRUE, 0);
-  gtk_widget_show_all ((GtkWidget *) auth.dbox->vbox);
+  gtk_widget_show_all (GTK_WIDGET (gtk_dialog_get_content_area (auth.dbox)));
 
   /*
    * event handling
