@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007 the xine-project
+ * Copyright (C) 2004-2017 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,13 +26,15 @@
 
 #include "globals.h"
 
-#include <xine.h>
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <math.h>
+
 #include <pango/pango-context.h>
 
+#include <xine.h>
+
 #include "post.h"
 #include "gtkvideo.h"
 #include "preferences.h"
@@ -55,7 +57,8 @@ typedef struct {
 
 struct gxine_post_s {
   gxine_chain_t *parent;
-  char *name, *parameters;	/* name, text config */
+  char *name;                   /* name */
+  gchar *parameters;            /* text config */
   xine_post_t *plugin;
   xine_post_in_t *info;
   GtkWidget *box, *enabled, *pref, *up;	/* in parent dbox */
@@ -76,21 +79,15 @@ static const char *const cfgname[] = {
   "gui.post_plugins.audio"
 };
 
-static const void *const empty[] = { NULL };
-
 static const char *const chain_title[] = {
   N_("Configure deinterlace post-plugins"),
   N_("Configure video post-plugins"),
   N_("Configure audio post-plugins")
 };
 
-static GtkTooltips *tips = NULL;
-
 static xine_video_port_t *vo_none = NULL;
 static xine_audio_port_t *ao_none = NULL;
 
-extern const char *xine_get_post_plugin_description () __attribute__((weak));
-
 static char *
 post_parse_parameters (xine_post_t *plugin, const char *paramstr)
 {
@@ -269,7 +266,7 @@ post_make_parameter_string (const gxine_post_t *info)
 {
   const xine_post_api_parameter_t *param =
     ((xine_post_api_t *)info->info->data)->get_param_descr ()->parameter - 1;
-  char *new_paramstr = strdup ("");
+  gchar *new_paramstr = g_strdup ("");
 
   while ((++param)->type != POST_PARAM_TYPE_LAST)
   {
@@ -418,17 +415,19 @@ spin_cb (GtkSpinButton *widget, int *param)
 static void
 range_cb (GtkAdjustment *widget, double *param)
 {
-  *param = widget->value;
+  *param = gtk_adjustment_get_value (widget);
 }
 
 static void
 help_realise_cb (GtkWidget *help, gpointer data)
 {
+  GtkStyle *style = gtk_widget_get_style (help);
   int i;
+
   for (i = 0; i < 5; ++i)
   {
-    gtk_widget_modify_text (help, i, help->style->fg + i);
-    gtk_widget_modify_base (help, i, help->style->bg + i);
+    gtk_widget_modify_text (help, i, style->fg + i);
+    gtk_widget_modify_base (help, i, style->bg + i);
   }
 }
 
@@ -469,10 +468,10 @@ check_box_cb (GtkToggleButton *widget, int *param)
 static void
 post_config_post_reconfigure (gxine_post_t *info)
 {
-  char *params = post_make_parameter_string (info);
+  gchar *params = post_make_parameter_string (info);
   logprintf ("post_config: plugin configuration\n  was: %s\n  now: %s\n",
 	     info->parameters, params);
-  free (info->parameters);
+  g_free (info->parameters);
   info->parameters = params;
 }
 
@@ -510,9 +509,10 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 		    G_CALLBACK(post_config_post_response), info);
   gtk_dialog_set_default_response (dbox, GTK_RESPONSE_OK);
   ui_add_undo_response (info->dbox, NULL);
-  free (title);
+  g_free (title);
   table = gtk_table_new (1, 2, FALSE);
-  gtk_box_pack_start (GTK_BOX(dbox->vbox), table, FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (dbox)),
+                      table, FALSE, TRUE, 0);
 
   info->params = post_parse_parameters (info->plugin, info->parameters);
   param = param_desc->parameter - 1;
@@ -529,9 +529,9 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
       if (param->enum_values)
       {
 	int i;
-	widget = gtk_combo_box_new_text ();
+        widget = gtk_combo_box_text_new ();
 	for (i = 0; param->enum_values[i]; ++i)
-	  gtk_combo_box_append_text (GTK_COMBO_BOX(widget),
+          gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT(widget),
 				     param->enum_values[i]);
 	gtk_combo_box_set_active (GTK_COMBO_BOX(widget), *(int *)param_p);
 	g_signal_connect (G_OBJECT(widget), "changed",
@@ -539,8 +539,8 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
       }
       else
       {
-	GtkObject *adj =
-	  gtk_adjustment_new (*(int *)param_p, INT_MIN, INT_MAX, 1, 10, 0);
+        GtkAdjustment *adj =
+          GTK_ADJUSTMENT (gtk_adjustment_new (*(int *)param_p, INT_MIN, INT_MAX, 1, 10, 0));
 	widget = ui_spin_button_new (adj);
 	g_signal_connect (G_OBJECT(widget), "value-changed",
 			  G_CALLBACK(spin_cb), param_p);
@@ -549,16 +549,16 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 
     case POST_PARAM_TYPE_DOUBLE:
       {
-	GtkObject *adj;
+        GtkAdjustment *adj;
 	gdouble range = param->range_max - param->range_min;
 	gboolean floating = range <= 10 || range != floor (range) ||
 			    param->range_min != floor (param->range_min);
 	gdouble step = pow (10, floor (log10 (range / 25)));
 	gdouble page = pow (10, floor (log10 (range / 10)));
 
-	adj = gtk_adjustment_new (*(double *)param_p,
-				  param->range_min, param->range_max,
-				  step, page, 0);
+        adj = GTK_ADJUSTMENT (gtk_adjustment_new (*(double *)param_p,
+                                                  param->range_min, param->range_max,
+                                                  step, page, 0));
 	widget = ui_hscale_new (adj, GTK_POS_TOP, floating ? 2 : 0);
 	gtk_widget_set_size_request (widget, 100, -1);
 	g_signal_connect (G_OBJECT (adj), "value-changed",
@@ -597,7 +597,7 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 		      1, 2, param_count, param_count + 1, 7, 7, 2, 2);
 
     if (param->description)
-      gtk_tooltips_set_tip (tips, widget, param->description, NULL);
+      gtk_widget_set_tooltip_text (widget, param->description);
 
     widget = ui_label_new_with_xalign (param->name, 1);
     gtk_table_attach (GTK_TABLE(table), widget,
@@ -654,10 +654,11 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
       gtk_text_view_set_wrap_mode ((GtkTextView *)help,
 				   wrap ? GTK_WRAP_WORD_CHAR : GTK_WRAP_NONE);
 
+      GtkStyle *style = gtk_widget_get_style (help);
       font = mono ? pango_font_description_from_string ("mono")
-		  : pango_font_description_copy_static (help->style->font_desc);
+                  : pango_font_description_copy_static (style->font_desc);
 
-      p = pango_font_description_get_size (help->style->font_desc);
+      p = pango_font_description_get_size (style->font_desc);
       pango_font_description_set_size (font, p * 5 / 6);
       gtk_widget_modify_font (help, font);
       pango_font_description_free (font);
@@ -670,12 +671,18 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 				      GTK_POLICY_AUTOMATIC);
 
       gtk_widget_set_size_request (window, 1, -1);
-      gtk_box_pack_end (GTK_BOX(dbox->vbox), window, TRUE, TRUE, 2);
+      gtk_box_pack_end (GTK_BOX (gtk_dialog_get_content_area (dbox)),
+                        window, TRUE, TRUE, 2);
     }
     free (helptext);
 
-    gtk_box_pack_end (GTK_BOX(dbox->vbox),
-		      gtk_hseparator_new (), FALSE, TRUE, 2);
+    gtk_box_pack_end (GTK_BOX (gtk_dialog_get_content_area (dbox)),
+#if GTK_CHECK_VERSION(3, 2, 0)
+                      gtk_separator_new (GTK_ORIENTATION_HORIZONTAL),
+#else
+                      gtk_hseparator_new (),
+#endif
+                      FALSE, TRUE, 2);
   }
 
   hide_on_delete (info->dbox, NULL);
@@ -737,6 +744,7 @@ reconfigure_plugins (gxine_chain_t *chain)
   free (params);
 }
 
+#if !GTK_CHECK_VERSION(3, 0, 0)
 typedef struct { GtkOptionMenu *menu; int index; } reset_pp_t;
 
 static gboolean reset_pp (gpointer data)
@@ -747,11 +755,32 @@ static gboolean reset_pp (gpointer data)
   return FALSE;
 }
 
+#else /* GTK_CHECK_VERSION(3, 0, 0) */
+typedef struct { GtkComboBox *menu; int index; } reset_pp_t;
+
+static gboolean reset_pp (gpointer data)
+{
+  reset_pp_t *info = data;
+  gtk_combo_box_set_active (info->menu, info->index);
+  free (info);
+  return FALSE;
+}
+#endif /* GTK_CHECK_VERSION(3, 0, 0) */
+
+#if !GTK_CHECK_VERSION(3, 0, 0)
 static void
 post_config_post_chain_set_plugin (GtkOptionMenu *menu, gxine_post_t *info)
+#else
+static void
+post_config_post_chain_set_plugin (GtkComboBox *menu, gxine_post_t *info)
+#endif
 {
   gxine_chain_t *chain = info->parent;
+#if !GTK_CHECK_VERSION(3, 0, 0)
   const char *name = chain->names[gtk_option_menu_get_history (menu)];
+#else
+  const char *name = chain->names[gtk_combo_box_get_active (menu)];
+#endif
   xine_post_t *newplugin;
 
   if (!strcmp (name, info->name))
@@ -771,7 +800,7 @@ post_config_post_chain_set_plugin (GtkOptionMenu *menu, gxine_post_t *info)
 	/* problem - we can't instantiate the plugin so we can't configure it */
 			 _("Couldn't initialise plugin ‘%s’ for configuration"),
 			 name);
-    if (info->name && strcmp (name, info->name))
+    if (strcmp (name, info->name))
     {
       int i;
       for (i = 0; chain->names[i]; ++i)
@@ -789,19 +818,16 @@ post_config_post_chain_set_plugin (GtkOptionMenu *menu, gxine_post_t *info)
   }
 
   free (info->name);
-  free (info->parameters);
+  g_free (info->parameters);
   xine_post_dispose (xine, info->plugin);
   info->name = strdup (name);
-  info->parameters = strdup ("");
+  info->parameters = g_strdup ("");
   info->plugin = newplugin;
   gtk_widget_set_sensitive
     (info->pref, xine_post_input (info->plugin, "parameters") != NULL);
 
-  gtk_tooltips_set_tip (tips, (GtkWidget *)menu,
-			xine_get_post_plugin_description
-			  ? xine_get_post_plugin_description (xine, name)
-			  : NULL,
-			NULL);
+  gtk_widget_set_tooltip_text (GTK_WIDGET (menu),
+                               xine_get_post_plugin_description (xine, name));
 }
 
 static void
@@ -813,9 +839,10 @@ post_config_post_chain_delete_plugin (GtkButton *widget, gxine_post_t *info)
       (G_OBJECT(info->dbox), G_CALLBACK(post_config_post_response), info);
   post_config_post_dispose_dbox (info);
   free (info->name);
-  free (info->parameters);
+  g_free (info->parameters);
   xine_post_dispose (xine, info->plugin);
-  gtk_container_remove (GTK_CONTAINER(info->box->parent), info->box);
+  gtk_container_remove (GTK_CONTAINER (gtk_widget_get_parent (info->box)),
+                        info->box);
   info->parent->plugins = g_slist_remove (info->parent->plugins, info);
   free (info);
 }
@@ -830,7 +857,8 @@ post_config_post_chain_move_plugin (GtkButton *widget, gxine_post_t *info)
   /* we're not removing the first item, but keep the compiler happy :-) */
   info->parent->plugins = g_slist_remove (info->parent->plugins, info);
   info->parent->plugins = g_slist_insert (info->parent->plugins, info, pos);
-  gtk_box_reorder_child (GTK_BOX(info->box->parent), info->box, pos);
+  gtk_box_reorder_child (GTK_BOX (gtk_widget_get_parent (info->box)),
+                         info->box, pos);
   gtk_widget_set_sensitive (info->up, pos);
   gtk_widget_set_sensitive
     (((gxine_post_t *)g_slist_nth_data (info->parent->plugins, pos + 1))->up,
@@ -842,7 +870,7 @@ post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
 			     gboolean enabled, char *name, char *params)
 {
   gxine_post_t *info = malloc (sizeof (gxine_post_t));
-  GtkWidget *w, *m;
+  GtkWidget *w;
   int i, history = 0;
   const char *hist_desc = NULL;
 
@@ -858,12 +886,18 @@ post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
   info->reconfigure = post_config_post_reconfigure;
   info->box = gtk_hbox_new (FALSE, 2);
 
+  info->enabled = w = gtk_check_button_new ();
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), enabled);
+  gtk_box_pack_start (GTK_BOX(info->box), w, FALSE, TRUE, 0);
+
+#if !GTK_CHECK_VERSION(3, 0, 0)
+
+  GtkWidget *m;
+
   m = gtk_menu_new ();
   for (i = 0; chain->names[i]; ++i)
   {
-    const char *desc = xine_get_post_plugin_description
-		     ? xine_get_post_plugin_description (xine, chain->names[i])
-		     : NULL;
+    const char *desc = xine_get_post_plugin_description (xine, chain->names[i]);
     GtkWidget *item;
     if (!strcmp (chain->names[i], name))
     {
@@ -873,24 +907,44 @@ post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
     gtk_menu_shell_append (GTK_MENU_SHELL(m),
 			item =   gtk_menu_item_new_with_label (chain->names[i]));
     if (desc)
-      gtk_tooltips_set_tip (tips, item, desc, NULL);
+      gtk_widget_set_tooltip_text (item, desc);
   }
 
-  info->enabled = w = gtk_check_button_new ();
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), enabled);
-  gtk_box_pack_start (GTK_BOX(info->box), w, FALSE, TRUE, 0);
-
-  GtkBox *box = GTK_BOX (info->box);
-
   /* FIXME but not until combo box menu items can have tooltips */
   w = gtk_option_menu_new ();
   gtk_option_menu_set_menu (GTK_OPTION_MENU(w), m);
   gtk_option_menu_set_history (GTK_OPTION_MENU(w), history);
-  gtk_box_pack_start_defaults (box, w);
+#else
+  w = gtk_combo_box_text_new ();
+
+  for (i = 0; chain->names[i]; ++i)
+  {
+    const char *desc = xine_get_post_plugin_description (xine, chain->names[i]);
+    GtkWidget *item;
+    if (!strcmp (chain->names[i], name))
+    {
+      history = i;
+      hist_desc = desc;
+    }
+    gtk_combo_box_text_append_text (w, chain->names[i]);
+    // XXX TODO
+    //if (desc)
+    //  gtk_widget_set_tooltip_text (item, desc);
+  }
+
+  /* FIXME but not until combo box menu items can have tooltips */
+
+  gtk_combo_box_set_active (GTK_COMBO_BOX(w), history);
+
+#endif
+
+  GtkBox *box = GTK_BOX (info->box);
+
+  gtk_box_pack_start (box, w, TRUE, TRUE, 0);
   g_signal_connect (G_OBJECT(w), "changed",
 		    G_CALLBACK(post_config_post_chain_set_plugin), info);
   if (hist_desc)
-    gtk_tooltips_set_tip (tips, w, hist_desc, NULL);
+    gtk_widget_set_tooltip_text (w, hist_desc);
 
   info->up = w = ui_button_new_stock (GTK_STOCK_GO_UP);
   gtk_widget_set_sensitive (w, !!chain->plugins); /* disable for first plugin */
@@ -912,8 +966,8 @@ post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
 					 G_CALLBACK(post_config_post_window),
 					 info);
 
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(chain->dbox)->vbox), info->box,
-		      FALSE, TRUE, 2);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (chain->dbox))),
+                      info->box, FALSE, TRUE, 2);
   gtk_widget_show_all (info->box);
 
   chain->plugins = g_slist_append (chain->plugins, info);
@@ -965,7 +1019,7 @@ post_config_post_chain_build (gxine_chain_t *chain)
       post_config_post_add_plugin (chain, post, enabled, name,
 				   g_strndup (first + 1, next - first - 1));
     else
-      free (name);
+      g_free (name);
 
     if (!*next)
       break;
@@ -1055,17 +1109,15 @@ post_config_post_chain_window (gxine_chain_t *chain)
 		   GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
 		   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 		   NULL);
-  free (title);
+  g_free (title);
   hide_on_delete (chain->dbox, NULL);
   g_signal_connect (G_OBJECT(chain->dbox), "response",
 		    G_CALLBACK(post_config_post_chain_response), chain);
   ui_add_undo_response (chain->dbox, NULL);
-  if (!tips)
-    tips = gtk_tooltips_new ();
 
   hbox = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_end (GTK_BOX(GTK_DIALOG(chain->dbox)->vbox), hbox,
-		    FALSE, TRUE, 5);
+  gtk_box_pack_end (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (chain->dbox))),
+                    hbox, FALSE, TRUE, 5);
 
   w = gtk_button_new_from_stock (GTK_STOCK_ADD);
   g_signal_connect (G_OBJECT(w), "clicked",
