@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2007 the xine project
+ * Copyright (C) 2001-2017 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,11 +24,6 @@
 
 #include "globals.h"
 
-#include <assert.h>
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkkeysyms.h>
-#include <glib.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -36,6 +31,14 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include <glib.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
+#if GTK_CHECK_VERSION(3, 0, 0)
+#include <gdk/gdkkeysyms-compat.h>
+#endif
+#include <gtk/gtk.h>
+
 #include "mediamarks.h"
 #include "preferences.h"
 #include "ui.h"
@@ -72,7 +75,6 @@ static GtkWidget  *prefs_dialog;
 static GNode	  *prefs = NULL;
 static GData	  *prefs_map = NULL;
 static int         is_visible;
-static GtkTooltips *tooltips = NULL;
 static int         update_lock = 0;
 
 
@@ -237,12 +239,13 @@ static void range_cb (GtkAdjustment *adj, const gchar *key)
 
   logprintf ("preferences: range cb for key %s\n", key);
 
+  double value = gtk_adjustment_get_value (adj);
   if (!xine_config_lookup_entry (xine, key, &entry) ||
-      entry.num_value == adj->value)
+      entry.num_value == value)
     return;
 
-  logprintf ("preferences: updating entry to %lf\n", adj->value);
-  entry.num_value = adj->value;
+  logprintf ("preferences: updating entry to %lf\n", value);
+  entry.num_value = value;
   xine_config_update_entry (xine, &entry);
 }
 
@@ -563,7 +566,10 @@ static void
 default_item_cb (GtkWidget *w, gpointer key)
 {
   xine_cfg_entry_t entry;
-  xine_config_lookup_entry (xine, key, &entry);
+
+  if (!xine_config_lookup_entry (xine, key, &entry))
+    return;
+
   if (entry.type == XINE_CONFIG_TYPE_STRING)
     entry.str_value = entry.str_default;
   else
@@ -602,9 +608,9 @@ create_item_default (const xine_cfg_entry_t * entry)
   }
 
   GtkWidget *widget = ui_button_new_stock (GTK_STOCK_CLEAR);
-  gtk_tooltips_set_tip (tooltips, widget, tip, NULL);
+  gtk_widget_set_tooltip_text (widget, tip);
   if (alloc)
-    free (tip);
+    g_free (tip);
 
   g_signal_connect ((GObject *) widget, "clicked",
 		    G_CALLBACK (default_item_cb), (gpointer) entry->key);
@@ -615,7 +621,10 @@ static void
 revert_item_cb (GtkWidget *w, gpointer key)
 {
   xine_cfg_entry_t entry;
-  xine_config_lookup_entry (xine, key, &entry);
+
+  if (!xine_config_lookup_entry (xine, key, &entry))
+    return;
+
   if (entry.type == XINE_CONFIG_TYPE_STRING)
     entry.str_value = g_object_get_data ((GObject *) w, "revert");
   else
@@ -629,6 +638,7 @@ create_item_revert (const xine_cfg_entry_t *entry)
   intptr_t value;
   char *tip;
   gboolean alloc = FALSE;
+  char *tmpstr = NULL;
 
   switch (entry->type)
   {
@@ -651,7 +661,8 @@ create_item_revert (const xine_cfg_entry_t *entry)
 
   case XINE_CONFIG_TYPE_STRING:
     tip = entry->str_value;
-    value = (intptr_t) strdup (entry->str_value);
+    tmpstr = strdup (entry->str_value);
+    value = (intptr_t) tmpstr;
     break;
 
   default:
@@ -662,9 +673,10 @@ create_item_revert (const xine_cfg_entry_t *entry)
   g_object_set_data ((GObject *) widget, "revert", (void *) value);
   g_signal_connect ((GObject *) widget, "clicked",
 		    G_CALLBACK (revert_item_cb), (gpointer) entry->key);
-  gtk_tooltips_set_tip (tooltips, widget, tip, NULL);
+  gtk_widget_set_tooltip_text (widget, tip);
   if (alloc)
-    free (tip);
+    g_free (tip);
+  free(tmpstr);
 
   return widget;
 }
@@ -678,13 +690,13 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
   case XINE_CONFIG_TYPE_ENUM:
     {
       int i;
-      widget = gtk_combo_box_new_text ();
+      widget = gtk_combo_box_text_new ();
       for (i = 0; entry->enum_values[i]; ++i)
       {
 	const char *label = gettext (entry->enum_values[i]);
 	if (label == entry->enum_values[i])
 	  label = dgettext (LIB_PACKAGE, label);
-	gtk_combo_box_append_text (GTK_COMBO_BOX(widget), label);
+        gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT(widget), label);
       }
       gtk_combo_box_set_active (GTK_COMBO_BOX(widget), entry->num_value);
       g_signal_connect (G_OBJECT(widget), "changed",
@@ -745,8 +757,8 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
 
   case XINE_CONFIG_TYPE_RANGE: /* slider */
     {
-      GtkObject *adj = gtk_adjustment_new (entry->num_value, entry->range_min,
-					   entry->range_max, 1.0, 10.0, 0.0);
+      GtkAdjustment *adj = GTK_ADJUSTMENT (gtk_adjustment_new (entry->num_value, entry->range_min,
+                                                               entry->range_max, 1.0, 10.0, 0.0));
       widget = ui_hscale_new (adj, GTK_POS_TOP, 0);
       gtk_widget_set_size_request (widget, 128, -1);
       g_signal_connect (adj, "value-changed",
@@ -756,8 +768,8 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
 
   case XINE_CONFIG_TYPE_NUM:
     {
-      GtkObject *adj = gtk_adjustment_new (entry->num_value, INT_MIN, INT_MAX,
-					   1, 10, 0);
+      GtkAdjustment *adj = GTK_ADJUSTMENT (gtk_adjustment_new (entry->num_value, INT_MIN, INT_MAX,
+                                                               1, 10, 0));
       widget = ui_spin_button_new (adj);
       g_signal_connect (G_OBJECT(widget), "value-changed",
 			G_CALLBACK(spin_cb), (gchar *)entry->key);
@@ -779,7 +791,7 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
   }
 
   g_datalist_set_data (&prefs_map, entry->key, widget);
-  gtk_tooltips_set_tip (tooltips, widget, entry->help, NULL);
+  gtk_widget_set_tooltip_text (widget, entry->help);
   return widget;
 }
 
@@ -802,7 +814,12 @@ static GtkWidget *create_item_label (const xine_cfg_entry_t *entry)
   gtk_label_set_line_wrap (GTK_LABEL(label), TRUE);
   gtk_label_set_line_wrap_mode (GTK_LABEL(label), PANGO_WRAP_WORD);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+#if GTK_CHECK_VERSION(3, 14, 0)
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_label_set_yalign (GTK_LABEL (label), 0.5);
+#else
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+#endif
 
   return label;
 }
@@ -832,7 +849,11 @@ static GNode *create_item (const xine_cfg_entry_t *entry)
   GtkWidget *extra = create_item_extra (entry);
   if (extra)
     g_object_set_data (G_OBJECT (item->editable), "extra", extra);
+#if GTK_CHECK_VERSION(3, 2, 0)
+  item->separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+#else
   item->separator = gtk_hseparator_new ();
+#endif
   item->exp = entry->exp_level;
   return g_node_new (item);
 }
@@ -909,7 +930,7 @@ static gboolean unmix_mixed_page (GNode *node, gboolean *done)
     /* found a mixed-mode page (this is normal) */
     prefix = g_strdup_printf ("%s. ", page->prefix);
     new_page = g_node_prepend (node, create_page (prefix, strlen (prefix)));
-    free (prefix);
+    g_free (prefix);
 
     /* move the new subpage's sibling item nodes into it */
     child = g_node_first_child (node);
@@ -935,22 +956,32 @@ static gboolean unmix_mixed_page (GNode *node, gboolean *done)
 static gboolean focus_item_cb (GtkWidget *widget, GtkDirectionType *dir,
 			       GNode *node)
 {
+  if (!node->parent)
+    return FALSE;
+
   const pref_t *pref = node->data;
-  const pref_t *parent = node->parent ? node->parent->data : NULL;
-  GtkWidget *viewport = parent->page.table->parent;
+  const pref_t *parent = node->parent->data;
+  GtkWidget *viewport = gtk_widget_get_parent (parent->page.table);
   GtkAdjustment *adj = gtk_viewport_get_vadjustment ((GtkViewport *)viewport);
 
-  int wt = pref->item.editable->allocation.y;
-  int wb = wt + pref->item.editable->allocation.height;
-  int lt = pref->item.label->allocation.y;
-  int lb = lt + pref->item.label->allocation.height;
+  GtkAllocation allocation_edit = { 0 };
+  GtkAllocation allocation_label = { 0 };
+  GtkAllocation allocation_viewport = { 0 };
+  gtk_widget_get_allocation (pref->item.editable, &allocation_edit);
+  gtk_widget_get_allocation (pref->item.label,    &allocation_label);
+  gtk_widget_get_allocation (viewport,            &allocation_viewport);
+
+  int wt = allocation_edit.y;
+  int wb = wt + allocation_edit.height;
+  int lt = allocation_label.y;
+  int lb = lt + allocation_label.height;
   int top = wt < lt ? wt : lt;
   int bottom = wb > lb ? wb : lb;
-
-  if (top < adj->value)
+  double value = gtk_adjustment_get_value (adj);
+  if (top < value)
     gtk_adjustment_set_value (adj, top);
-  else if (bottom + 4 > adj->value + viewport->allocation.height)
-    gtk_adjustment_set_value (adj, bottom - viewport->allocation.height + 4);
+  else if (bottom + 4 > value + allocation_viewport.height)
+    gtk_adjustment_set_value (adj, bottom - allocation_viewport.height + 4);
 
   return FALSE;
 }
@@ -987,14 +1018,14 @@ static gboolean put_content (GNode *node, gpointer data)
     if (parent)
     {
       /* add this page to its parent's notebook */
-      gtk_widget_ref (pref->page.page); /* hiding is done by removing */
+      g_object_ref (pref->page.page); /* hiding is done by removing */
       gtk_notebook_append_page (GTK_NOTEBOOK(parent->notebook), pref->page.page,
 				gtk_label_new (pref->page.label));
     }
     else
       gtk_widget_set_size_request (pref->page.page, 600, 350);
   }
-  else
+  else if (parent)
   {
     /* we have a config item */
     int r = g_node_child_position (node->parent, node) * 2;
@@ -1009,8 +1040,8 @@ static gboolean put_content (GNode *node, gpointer data)
       GtkBox *box = GTK_BOX (gtk_vbox_new (2, FALSE));
       gtk_table_attach (table, (GtkWidget *) box, 2, 3, r, r + 1,
 			GTK_EXPAND | GTK_FILL, GTK_SHRINK, 2, 5);
-      gtk_box_pack_start_defaults (box, pref->item.editable);
-      gtk_box_pack_start_defaults (box, extra);
+      gtk_box_pack_start (box, pref->item.editable, TRUE, TRUE, 0);
+      gtk_box_pack_start (box, extra, TRUE, TRUE, 0);
 #ifdef GXINE_DEVICE_INFO
       xine_cfg_entry_t entry;
       const char *key = g_object_get_data (G_OBJECT (pref->item.editable), "cfg");
@@ -1044,7 +1075,6 @@ static GtkWidget *make_prefs_window (void)
   if (!xine_config_get_first_entry (xine, &entry))
     return NULL;
 
-  tooltips = gtk_tooltips_new ();
   prefs = create_page ("", 0); /* root page */
 
   do
@@ -1089,7 +1119,6 @@ static GtkWidget *make_prefs_window (void)
 
   g_node_traverse (prefs, G_PRE_ORDER, G_TRAVERSE_ALL, -1, put_content, NULL);
 
-  gtk_tooltips_enable (tooltips);
   return PREF_PAGE_T(prefs)->page;
 }
 
@@ -1131,14 +1160,14 @@ static void preferences_init_dbox (void)
 
   /* Make new tabbed box (notebook) */
   GtkWidget *prefs_notebook = make_prefs_window ();
-  gtk_box_pack_start_defaults (GTK_BOX(GTK_DIALOG(prefs_dialog)->vbox),
-			       prefs_notebook);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (prefs_dialog))),
+                      prefs_notebook, TRUE, TRUE, 0);
 
   GtkWidget *w = ui_label_new_with_markup
 		   (_("Items marked “<b>• <i>like this</i></b>” require "
 		      "gxine to be restarted to take effect."));
-  gtk_box_pack_end (GTK_BOX(GTK_DIALOG(prefs_dialog)->vbox), w,
-		    FALSE, FALSE, 2);
+  gtk_box_pack_end (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (prefs_dialog))),
+                    w, FALSE, FALSE, 2);
 }
 
 static gpointer preferences_first_show (gpointer unused)
@@ -1146,7 +1175,7 @@ static gpointer preferences_first_show (gpointer unused)
   static int value = 0;
   xine_cfg_entry_t entry;
 
-  gtk_widget_show_all (GTK_DIALOG (prefs_dialog)->vbox);
+  gtk_widget_show_all (gtk_dialog_get_content_area (GTK_DIALOG (prefs_dialog)));
   if (xine_config_lookup_entry (xine, "gui.experience_level", &entry))
     value = entry.num_value;
 
