@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2007 the xine project
+ * Copyright (C) 2003-2017 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,9 @@
 #include <stdarg.h>
 
 #include <gdk/gdkkeysyms.h>
+#if GTK_CHECK_VERSION(3, 0, 0)
+#include <gdk/gdkkeysyms-compat.h>
+#endif
 
 #include "script_engine.h"
 #include "ui.h"
@@ -96,7 +99,7 @@ static void se_store_text (const char *type, const char *text)
 
 static void se_js_icon (int flag)
 {
-  if (!GTK_WIDGET_VISIBLE (se_window))
+  if (!gtk_widget_get_visible (se_window))
   {
     int oldv = se_prop_get_int (gse, js_icon_obj, "v");
     int newv = oldv | flag;
@@ -128,32 +131,32 @@ void se_error_cb (const char *msg, const JSErrorReport *report)
   };
   static const char flag[] = { 8, 4, 8, 2 };
 
-  char *message = g_markup_printf_escaped ("<i>%s</i>", msg);
+  gchar *message = g_markup_printf_escaped ("<i>%s</i>", msg);
   ssize_t l = strlen (message) - 4;
   while (--l >= 0 &&
 	 (message[l] == ' ' || message[l] == '\n' || message[l] == '\t'))
     /**/;
   strcpy (message + l + 1, "</i>"); /* may have been chopped off */
-  char *file = report->filename
+  gchar *file = report->filename
 	     ? g_markup_printf_escaped (_("<b>Source:</b> %s"), report->filename)
 	     : NULL;
-  char *line = report->lineno
+  gchar *line = report->lineno
 	     ? g_markup_printf_escaped (_("<b>Line:</b> %d"), report->lineno)
 	     : NULL;
-  char *stmt = report->linebuf
+  gchar *stmt = report->linebuf
 	     ? g_markup_printf_escaped (_("<b>Line:</b> %s"), report->linebuf)
 	     : NULL;
 
-  char *text = g_strconcat (message, file ? "\n" : "", file ? : "",
+  gchar *text = g_strconcat (message, file ? "\n" : "", file ? : "",
 			    line ? "\n" : "", line ? : "",
 			    stmt ? "\n" : "", stmt ? : "", NULL);
 
-  free (message);
-  free (line);
-  free (file);
-  free (stmt);
+  g_free (message);
+  g_free (line);
+  g_free (file);
+  g_free (stmt);
   se_store_text (type[report->flags & 3], text);
-  free (text);
+  g_free (text);
   se_js_icon (flag[report->flags & 3]);
 }
 
@@ -304,7 +307,7 @@ static JSBool controls_exit (JSContext *cx, uintN argc, jsval *vp)
 static JSBool show_js_console (JSContext *cx, uintN argc, jsval *vp)
 {
   se_log_fncall_checkinit ("show_js_console");
-  if (GTK_WIDGET_VISIBLE (se_window))
+  if (gtk_widget_get_visible (se_window))
     gtk_widget_hide (se_window);
   else
   {
@@ -493,7 +496,7 @@ static char *show_help_int (se_t *se, se_o_t *obj, se_group_t selector)
 
 static JSBool show_help (JSContext *cx, uintN argc, jsval *vp)
 {
-  static char *const group_id[] = {
+  static const char *const group_id[] = {
     NULL,
     N_("Engine control:"),
     N_("Dialogue boxes:"),
@@ -894,26 +897,26 @@ static JSBool generic_Constructor (JSContext* cx, uintN argc, jsval* vp)
 }
 #endif
 
-static JSClass generic_JSClass =
-  {
-    "view", JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub,
-    generic_JSGetProperty, generic_JSSetProperty,
-    JS_EnumerateStub, JS_ResolveStub,
-#ifndef JSFUN_CONSTRUCTOR
-    JS_ConvertStub, generic_JSDestructor
-  };
-#else
-    JS_ConvertStub, generic_JSDestructor,
-    NULL, NULL, NULL, generic_Constructor,
-    NULL, NULL, NULL, NULL
-  };
-#endif
-
 se_o_t *se_create_object (se_t *se, se_o_t *parent /* may be NULL */,
 			  const gchar *name, void *user_data,
 			  se_group_t group, const gchar *help)
 {
+  static JSClass generic_JSClass = {
+    .name        = "view",
+    .flags       = JSCLASS_HAS_PRIVATE,
+    .addProperty = JS_PropertyStub,
+    .delProperty = JS_PropertyStub,
+    .getProperty = generic_JSGetProperty,
+    .setProperty = generic_JSSetProperty,
+    .enumerate   = JS_EnumerateStub,
+    .resolve     = JS_ResolveStub,
+    .convert     = JS_ConvertStub,
+    .finalize    = generic_JSDestructor,
+#ifdef JSFUN_CONSTRUCTOR
+    .construct   = generic_Constructor,
+#endif
+  };
+
   se_o_t *o;
 
   logprintf ("script_engine: creating object %s parent: %s\n",
@@ -975,7 +978,7 @@ se_o_t *se_find_object (se_t *se, se_o_t *parent, const char *name)
   return NULL;
 }
 
-static se_o_t dflt = {};
+static se_o_t dflt;
 
 se_o_t *se_get_object (se_t *se, se_o_t *parent, const char *name)
 {
@@ -1333,7 +1336,11 @@ void se_prop_create_xine_id (se_t *se, se_o_t *o,
 			     const char *id, const char *xine_id)
 {
   xine_cfg_entry_t entry = { NULL };
-  xine_config_lookup_entry (xine, xine_id, &entry);
+
+  if (!xine_config_lookup_entry (xine, xine_id, &entry)) {
+    entry.type = XINE_CONFIG_TYPE_UNKNOWN;
+  }
+
   switch (entry.type)
   {
   case XINE_CONFIG_TYPE_RANGE:
@@ -1488,7 +1495,7 @@ static gboolean
 console_scroll_cb (GtkWidget *w, GdkEventKey *e, GtkWidget *t)
 {
   GtkAdjustment *adj = gtk_tree_view_get_vadjustment (tree_view);
-  gdouble value = adj->value;
+  gdouble value = gtk_adjustment_get_value (adj);
 
   switch (e->state & GXINE_MODIFIER_MASK)
   {
@@ -1499,8 +1506,8 @@ console_scroll_cb (GtkWidget *w, GdkEventKey *e, GtkWidget *t)
     switch (e->keyval)
     {
     default:		return FALSE;
-    case GDK_Page_Up:	value -= adj->page_increment; break;
-    case GDK_Page_Down:	value += adj->page_increment; break;
+    case GDK_Page_Up:   value -= gtk_adjustment_get_page_increment (adj); break;
+    case GDK_Page_Down: value += gtk_adjustment_get_page_increment (adj); break;
     }
     break;
 
@@ -1508,10 +1515,10 @@ console_scroll_cb (GtkWidget *w, GdkEventKey *e, GtkWidget *t)
     switch (e->keyval)
     {
     default:		return FALSE;
-    case GDK_Up:	value -= adj->step_increment; break;
-    case GDK_Down:	value += adj->step_increment; break;
-    case GDK_Page_Up:	value -= adj->page_increment; break;
-    case GDK_Page_Down:	value += adj->page_increment; break;
+    case GDK_Up:        value -= gtk_adjustment_get_step_increment (adj); break;
+    case GDK_Down:      value += gtk_adjustment_get_step_increment (adj); break;
+    case GDK_Page_Up:   value -= gtk_adjustment_get_page_increment (adj); break;
+    case GDK_Page_Down: value += gtk_adjustment_get_page_increment (adj); break;
     }
     break;
 
@@ -1519,16 +1526,16 @@ console_scroll_cb (GtkWidget *w, GdkEventKey *e, GtkWidget *t)
     switch (e->keyval)
     {
     default:		return FALSE;
-    case GDK_Page_Up:	value = adj->lower; break;
-    case GDK_Page_Down:	value = adj->upper; break;
+    case GDK_Page_Up:   value = gtk_adjustment_get_lower (adj); break;
+    case GDK_Page_Down: value = gtk_adjustment_get_upper (adj); break;
     }
     break;
   }
 
-  if (value < adj->lower)
-    value = adj->lower;
-  else if (value > adj->upper - adj->page_size)
-    value = adj->upper - adj->page_size;
+  if (value < gtk_adjustment_get_lower(adj))
+    value = gtk_adjustment_get_lower (adj);
+  else if (value > gtk_adjustment_get_upper(adj) - gtk_adjustment_get_page_size(adj))
+    value = gtk_adjustment_get_upper(adj) - gtk_adjustment_get_page_size(adj);
 
   gtk_adjustment_set_value (adj, value);
   return TRUE;
@@ -1552,11 +1559,11 @@ se_create_console (void)
 	NULL);
 
   w = gtk_hbox_new (FALSE, 2);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (se_window)->vbox), w,
-		      FALSE, FALSE, 2);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (se_window))),
+                      w, FALSE, FALSE, 2);
 
   eval_box = history_combo_box_new (se_execute_cb);
-  gtk_box_pack_start_defaults (GTK_BOX (w), eval_box);
+  gtk_box_pack_start (GTK_BOX (w), eval_box, TRUE, TRUE, 0);
 
   b = gtk_button_new_from_stock (GTK_STOCK_EXECUTE);
   gtk_box_pack_start (GTK_BOX (w), b, FALSE, FALSE, 0);
@@ -1595,18 +1602,27 @@ se_create_console (void)
   g_signal_connect (G_OBJECT (eval_box), "key-press-event",
 		    G_CALLBACK (console_scroll_cb), w);
 
-  gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (se_window)->vbox), w);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (se_window))),
+                      w, TRUE, TRUE, 0);
 }
 
 se_t *se_new (void)
 {
-  se_t    *se;
   static JSClass global_class = {
-    "global", JSCLASS_HAS_PRIVATE | JSCLASS_GLOBAL_FLAGS,
-    JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,JS_PropertyStub,
-    JS_EnumerateStub,JS_ResolveStub,JS_ConvertStub,JS_FinalizeStub
+    .name        = "global",
+    .flags       = JSCLASS_HAS_PRIVATE | JSCLASS_GLOBAL_FLAGS,
+    .addProperty = JS_PropertyStub,
+    .delProperty = JS_PropertyStub,
+    .getProperty = JS_PropertyStub,
+    .setProperty = JS_StrictPropertyStub,
+    .enumerate   = JS_EnumerateStub,
+    .resolve     = JS_ResolveStub,
+    .convert     = JS_ConvertStub,
+    .finalize    = JS_FinalizeStub,
   };
 
+  se_t    *se;
+
   /*
    * create the JS console window
    */
