@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007 the xine-project
+ * Copyright (C) 2004-2017 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,9 +26,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
 #include <pthread.h>
+
+#include <gdk/gdkkeysyms.h>
+#if GTK_CHECK_VERSION(3, 0, 0)
+#include <gdk/gdkkeysyms-compat.h>
+#endif
+#include <gtk/gtk.h>
+
 /* for VO_CAP_HUE etc. (where available) */
 #include <xine/video_out.h>
 
@@ -64,7 +69,7 @@ static ui_status_t status = UI_CURRENT_STATE;
 pthread_mutex_t status_lock = PTHREAD_MUTEX_INITIALIZER;
 
 static GSList *c_buttons[Control_Buttons] = { NULL };
-static GtkObject *c_adjustments[Control_Adjustments] = { NULL };
+static GtkAdjustment *c_adjustments[Control_Adjustments] = { NULL };
 static GSList *c_adj_widgets[Control_Adjustments] = { NULL };
 
 /* (Internal) Reflect play/pause/stop status in control buttons */
@@ -179,6 +184,7 @@ typedef struct {
 static const ui_adjustable_t ranges[] = {
   [Control_SEEKER]		= {     0,       0,  65535,    1,    10, 0, NULL },
   [Control_AUDIO_CHANNEL]	= {    -1,      -1,     32,    1,     1, 0, NULL,			 "ao_channel",	  XINE_PARAM_AUDIO_CHANNEL_LOGICAL },
+  [Control_SPU_CHANNEL]	        = {    -1,      -1,     32,    1,     1, 0, NULL,			 "spu_channel",	  XINE_PARAM_SPU_CHANNEL },
   [Control_VOLUME]		= {    50,       0,    100,    1,    10, 0, "audio.volume.mixer_volume", "ao_volume",	  XINE_PARAM_AUDIO_VOLUME },
   [Control_COMPRESSOR]		= {   100,     100,   1000,   10,   100, 0, "gui.ao_compressor",	 "ao_compressor", XINE_PARAM_AUDIO_COMPR_LEVEL },
   [Control_AMPLIFIER]		= {   100,       0,    200,    1,    10, 0, "gui.ao_amplifier",		 "ao_amplifier",  XINE_PARAM_AUDIO_AMP_LEVEL },
@@ -229,14 +235,14 @@ static void ui_adjustment_value_changed_cb (GtkAdjustment *adj,
     return;
 
   if (info->param)
-    xine_set_param (stream, info->param, adj->value);
+    xine_set_param (stream, info->param, gtk_adjustment_get_value(adj));
 
   if (info->param == XINE_PARAM_AUDIO_VOLUME)
     return;
   if (info->setting &&
       xine_config_lookup_entry (xine, info->setting, &entry))
   {
-    entry.num_value = adj->value;
+    entry.num_value = gtk_adjustment_get_value(adj);
     preferences_update_entry (&entry);
   }
 }
@@ -250,13 +256,13 @@ ui_seeker_value_changed_cb (GtkAdjustment *seeker, gpointer data)
     int speed;
     ++no_recursion;
     speed = xine_get_param (stream, XINE_PARAM_SPEED);
-    xine_play (stream, (gint) seeker->value, 0);
+    xine_play (stream, (gint) gtk_adjustment_get_value(seeker), 0);
     xine_set_param (stream, XINE_PARAM_SPEED, speed);
     --no_recursion;
   }
 }
 
-GtkObject *ui_register_control_adjustment (control_adjustment_t item)
+GtkAdjustment *ui_register_control_adjustment (control_adjustment_t item)
 {
   g_return_val_if_fail (item < Control_Adjustments, NULL);
 
@@ -276,9 +282,9 @@ GtkObject *ui_register_control_adjustment (control_adjustment_t item)
     starts[item] = start;
 
     c_adjustments[item] =
-      gtk_adjustment_new (start, ranges[item].min, ranges[item].max,
-			  ranges[item].step, ranges[item].page,
-			  ranges[item].pagesize);
+      GTK_ADJUSTMENT (gtk_adjustment_new (start, ranges[item].min, ranges[item].max,
+                                          ranges[item].step, ranges[item].page,
+                                          ranges[item].pagesize));
     if (ranges[item].param || ranges[item].setting)
       g_signal_connect (c_adjustments[item], "value-changed",
 			G_CALLBACK(ui_adjustment_value_changed_cb),
@@ -306,7 +312,7 @@ void ui_set_control_button (control_button_t item, gboolean state)
   g_return_if_fail (item >= Control_PlayerButtons && item < Control_Buttons);
   no_recursion = TRUE;
   if (c_buttons[item] &&
-      (GTK_TOGGLE_BUTTON(c_buttons[item]->data))->active != state)
+      (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (c_buttons[item]->data)) != state))
     foreach_glist (btn, c_buttons[item])
       g_object_set (G_OBJECT(btn->data), "active", state, NULL);
   no_recursion = FALSE;
@@ -318,7 +324,7 @@ void ui_set_control_adjustment (control_adjustment_t item, gdouble value)
 {
   g_return_if_fail (item < Control_Adjustments);
   no_recursion = TRUE;
-  if ((GTK_ADJUSTMENT(c_adjustments[item]))->value != value)
+  if (gtk_adjustment_get_value (GTK_ADJUSTMENT (c_adjustments[item])) != value)
     g_object_set (G_OBJECT(c_adjustments[item]), "value", value, NULL);
   no_recursion = FALSE;
 }
@@ -356,7 +362,14 @@ void ui_xine_set_param_from_adjustment (control_adjustment_t item)
 static GtkWidget *ui_container_new_stock (GtkWidget *box, const char *stock)
 {
   GtkWidget *img = gtk_image_new_from_stock (stock, GTK_ICON_SIZE_BUTTON);
+#if GTK_CHECK_VERSION(3, 14, 0)
+  gtk_widget_set_margin_start (img, 0);
+  gtk_widget_set_margin_end (img, 0);
+  gtk_widget_set_margin_top (img, 0);
+  gtk_widget_set_margin_bottom (img, 0);
+#else
   gtk_misc_set_padding (GTK_MISC(img), 0, 0);
+#endif
   if (box)
     gtk_container_add (GTK_CONTAINER(box), img);
   return box ? box : img;
@@ -391,7 +404,14 @@ GtkWidget *ui_button_new_stock_mnemonic (const char *stock,
 static GtkWidget *ui_container_new_icon_name (GtkWidget *box, const char *name)
 {
   GtkWidget *img = gtk_image_new_from_icon_name (name, GTK_ICON_SIZE_BUTTON);
+#if GTK_CHECK_VERSION(3, 14, 0)
+  gtk_widget_set_margin_start (img, 0);
+  gtk_widget_set_margin_end (img, 0);
+  gtk_widget_set_margin_top (img, 0);
+  gtk_widget_set_margin_bottom (img, 0);
+#else
   gtk_misc_set_padding (GTK_MISC(img), 0, 0);
+#endif
   if (box)
     gtk_container_add (GTK_CONTAINER(box), img);
   return box ? box : img;
@@ -425,14 +445,14 @@ GtkWidget *ui_button_new_icon_name_mnemonic (const char *name,
 
 /* Misc widgets with common properties */
 
-GtkWidget *ui_spin_button_new (GtkObject *adj)
+GtkWidget *ui_spin_button_new (GtkAdjustment *adj)
 {
   return g_object_new (GTK_TYPE_SPIN_BUTTON, "adjustment", adj, "digits", 0,
 		       "update-policy", GTK_UPDATE_ALWAYS, "climb-rate", 1.0,
 		       "numeric", TRUE, NULL);
 }
 
-GtkWidget *ui_hscale_new (GtkObject *adj, GtkPositionType pos, int digits)
+GtkWidget *ui_hscale_new (GtkAdjustment *adj, GtkPositionType pos, int digits)
 {
   return g_object_new (GTK_TYPE_HSCALE, "adjustment", adj, "value-pos", pos,
 		       "digits", digits, NULL);
@@ -736,16 +756,17 @@ static void post_plugins_audio_cb (void *data, xine_cfg_entry_t *cfg)
 
 void ui_preferences_register (xine_t *this)
 {
-  static const char *experience_labels[] = {
+  static const char * const experience_labels[] = {
     N_("Beginner"), N_("Advanced"), N_("Expert"),
     N_("Master of the known universe"), NULL
   };
-  static const char *tbar_pos_labels[] = {
+  static const char * const tbar_pos_labels[] = {
     N_("Top, hidden"), N_("Bottom, hidden"),
     N_("Top, visible"), N_("Bottom, visible"),
     NULL
   };
-  const char **vis_labels, *const *vis_src;
+  const char **vis_labels;
+  const char * const *vis_src;
   unsigned int i;
 
   config_version = xine_config_register_num
@@ -759,7 +780,7 @@ void ui_preferences_register (xine_t *this)
 
   /* Register the experience level setting */
   xine_config_register_enum
-    (this, "gui.experience_level", 0, experience_labels,
+    (this, "gui.experience_level", 0, (char **)experience_labels,
      _("Display of configuration settings"),
      _("Controls whether more advanced configuration settings are shown."),
      0, GXINE_TRUE, NULL);
@@ -822,7 +843,7 @@ void ui_preferences_register (xine_t *this)
   memcpy (vis_labels + 1, vis_src, (i + 1) * sizeof (char *));
 
   xine_config_register_enum
-    (this, "gui.post_plugins.audio_visualisation", goom, vis_labels,
+    (this, "gui.post_plugins.audio_visualisation", goom, (char **)vis_labels,
      _("Default audio visualisation plugin"),
      _("Post-plugin to be used when playing streams without video"),
      10, NULL, NULL);
@@ -838,7 +859,7 @@ void ui_preferences_register (xine_t *this)
      NULL, 0, NULL, NULL);
 
   xine_config_register_enum
-    (this, "gui.fullscreen_mode.toolbar", 1, tbar_pos_labels,
+    (this, "gui.fullscreen_mode.toolbar", 1, (char **)tbar_pos_labels,
      _("Default position & visibility of the full-screen toolbar"),
      NULL, 0, NULL, NULL);
 
@@ -947,7 +968,7 @@ static void ui_load_icons (void)
   int i;
   for (i = 0; i < G_N_ELEMENTS (icons); ++i)
   {
-	char name[128];
+    char name[128];
     snprintf (name, sizeof(name), "%s.%s", icons[i][1], fmt ? "svg" : "png");
     ui_std_icon (ifactory, icons[i][0], name);
   }
@@ -963,14 +984,16 @@ static void ui_load_icons (void)
 
 /* GtkEntry clipboard paste and button press overrides */
 
-static void (*old_gtk_entry_paste) (GtkEntry *, gpointer);
-static gint (*old_gtk_entry_click) (GtkWidget *, GdkEventButton *, gpointer);
+static void (*old_gtk_entry_paste) (GtkEntry *);
+#if !GTK_CHECK_VERSION(3, 0, 0)
+static gint (*old_gtk_entry_click) (GtkWidget *, GdkEventButton *);
+#endif
 
-static void ui_paste_clipboard_cb (GtkEntry *entry, gpointer data)
+static void ui_paste_clipboard_cb (GtkEntry *entry)
 {
-  if (entry->editable)
+  GtkEditable *edit = GTK_EDITABLE (entry);
+  if (gtk_editable_get_editable(edit))
   {
-    GtkEditable *edit = GTK_EDITABLE (entry);
     play_item_t *item = clip_get_play_item ();
     if (item)
     {
@@ -994,12 +1017,13 @@ static void ui_paste_clipboard_cb (GtkEntry *entry, gpointer data)
       play_item_dispose (item);
     }
     else
-      old_gtk_entry_paste (entry, data);
+      old_gtk_entry_paste (entry);
   }
 }
 
-static gint ui_click_clipboard_cb (GtkWidget *widget, GdkEventButton *event,
-				   gpointer data)
+#if !GTK_CHECK_VERSION(3, 0, 0)
+/* XXX this code hasn't been ported to Gtk 3 */
+static gint ui_click_clipboard_cb (GtkWidget *widget, GdkEventButton *event)
 {
   GtkEntry *entry = (GtkEntry *) widget;
 
@@ -1024,8 +1048,9 @@ static gint ui_click_clipboard_cb (GtkWidget *widget, GdkEventButton *event,
     /* if we reach here, nothing has been pasted... */
   }
 
-  return old_gtk_entry_click (widget, event, data);
+  return old_gtk_entry_click (widget, event);
 }
+#endif /* !GTK_CHECK_VERSION(3, 0, 0) */
 
 /* UI code init */
 
@@ -1073,8 +1098,10 @@ void ui_init (void)
     GtkWidgetClass *wclass = (GtkWidgetClass *) klass;
     old_gtk_entry_paste = klass->paste_clipboard;
     klass->paste_clipboard = ui_paste_clipboard_cb;
+#if !GTK_CHECK_VERSION(3, 0, 0)
     old_gtk_entry_click = wclass->button_press_event;
     wclass->button_press_event = ui_click_clipboard_cb;
+#endif
   }
 
   toolbars = se_create_object (gse, NULL, "toolbar", NULL,
@@ -1117,7 +1144,7 @@ void ui_init (void)
   }
 }
 
-gboolean ui_post_init (void)
+void ui_post_init (void)
 {
   unsigned int i;
   uint32_t cap = gtk_video_get_capabilities ((GtkVideo *)gtv);
@@ -1144,9 +1171,6 @@ gboolean ui_post_init (void)
 #endif
   }
   gdk_threads_leave ();
-
-  gxine_init_done ();
-  return FALSE;
 }
 
 /* Undo button */
