@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2008 the xine-project
+ * Copyright (C) 2001-2017 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -289,18 +289,18 @@ char *get_config_filename (const char *leaf)
 
 FILE *open_write (const char *file, const char *errtitle)
 {
-  char *fname = g_strconcat (file, ".new", NULL);
+  gchar *fname = g_strconcat (file, ".new", NULL);
   FILE *fd = NULL;
   if ((unlink (fname) && errno != ENOENT) || !(fd = fopen (fname, "w")))
     display_error_modal (FROM_GXINE, errtitle, _("Can't open file ‘%s’: %s"),
 			 fname, strerror (errno));
-  free (fname);
+  g_free (fname);
   return fd;
 }
 
 void close_write (const char *file, FILE *fd, const char *errtitle)
 {
-  char *fname = g_strconcat (file, ".new", NULL);
+  gchar *fname = g_strconcat (file, ".new", NULL);
   int error;
 
   if ((error = ferror (fd)))
@@ -318,6 +318,8 @@ void close_write (const char *file, FILE *fd, const char *errtitle)
     display_error_modal (FROM_GXINE, errtitle,
 			 _("Error when replacing ‘%s’: %s"),
 			 file, strerror (errno));
+
+  g_free (fname);
 }
 
 void window_show (GtkWidget *const widget, GtkWidget *const parent)
@@ -328,7 +330,7 @@ void window_show (GtkWidget *const widget, GtkWidget *const parent)
   gtk_window_set_transient_for ((GtkWindow *)widget,
 				(GtkWindow *)(parent ? : app));
   gtk_widget_show_all (widget);
-  gdk_window_raise (widget->window);
+  gdk_window_raise (gtk_widget_get_window (widget));
 }
 
 void window_present (GtkWidget *const widget, GtkWidget *const parent)
@@ -339,7 +341,7 @@ void window_present (GtkWidget *const widget, GtkWidget *const parent)
   gtk_window_set_transient_for ((GtkWindow *)widget,
 				(GtkWindow *)(parent ? : app));
   gtk_widget_show (widget);
-  gdk_window_raise (widget->window);
+  gdk_window_raise (gtk_widget_get_window (widget));
 }
 
 static void response_cb (GtkWidget *dialog, gint response, gpointer data)
@@ -380,7 +382,6 @@ GtkWidget *v_display_message (gxine_msg_source src, gboolean modal,
 
   GtkWidget *dialog;
   gchar     *msg = g_strdup_vprintf (fmt, ap);
-  va_end (ap);
 
   switch (type)
   {
@@ -463,6 +464,7 @@ void display_error_modal (gxine_msg_source src, const gchar *title,
   va_list ap;
   va_start (ap, fmt);
   v_display_message (src, TRUE, title, GTK_MESSAGE_ERROR, fmt, ap);
+  va_end (ap);
 }
 
 void display_error (gxine_msg_source src, const gchar *title,
@@ -471,6 +473,7 @@ void display_error (gxine_msg_source src, const gchar *title,
   va_list ap;
   va_start (ap, fmt);
   v_display_message (src, FALSE, title, GTK_MESSAGE_ERROR, fmt, ap);
+  va_end (ap);
 }
 
 void display_warning (gxine_msg_source src, const gchar *title,
@@ -479,6 +482,7 @@ void display_warning (gxine_msg_source src, const gchar *title,
   va_list ap;
   va_start (ap, fmt);
   v_display_message (src, FALSE, title, GTK_MESSAGE_WARNING, fmt, ap);
+  va_end (ap);
 }
 
 void display_info (gxine_msg_source src, const gchar *title,
@@ -487,6 +491,7 @@ void display_info (gxine_msg_source src, const gchar *title,
   va_list ap;
   va_start (ap, fmt);
   v_display_message (src, FALSE, title, GTK_MESSAGE_INFO, fmt, ap);
+  va_end (ap);
 }
 
 static void destroy_combinable (GtkWidget *dbox, gint which, gpointer data)
@@ -507,13 +512,20 @@ v_display_combinable (gxine_msg_source src, const gchar *title,
   va_list apx;
   gchar *msg;
   va_copy (apx, ap);
-  vasprintf (&msg, fmt, ap);
-  msg = title ? g_markup_printf_escaped ("<b>%s</b>\n%s", title, msg)
-	      : strdup (msg);
-  combinable.dbox = v_display_message (src, FALSE, title, type, fmt, apx);
-  combinable.text = msg;
-  g_signal_connect (G_OBJECT (combinable.dbox), "response",
-		    G_CALLBACK (destroy_combinable), msg);
+
+  msg = g_strdup_vprintf (fmt, ap);
+
+  if (msg) {
+
+    if (title)
+      asreprintf(&msg, "<b>%s</b>\n%s", title, msg);
+
+    combinable.dbox = v_display_message (src, FALSE, title, type, fmt, apx);
+    combinable.text = msg;
+    g_signal_connect (G_OBJECT (combinable.dbox), "response",
+                      G_CALLBACK (destroy_combinable), msg);
+  }
+  va_end (apx);
 }
 
 void display_error_combinable (gxine_msg_source src, const gchar *title,
@@ -522,6 +534,7 @@ void display_error_combinable (gxine_msg_source src, const gchar *title,
   va_list ap;
   va_start (ap, fmt);
   v_display_combinable (src, title, GTK_MESSAGE_ERROR, fmt, ap);
+  va_end (ap);
 }
 
 void display_warning_combinable (gxine_msg_source src, const gchar *title,
@@ -530,6 +543,7 @@ void display_warning_combinable (gxine_msg_source src, const gchar *title,
   va_list ap;
   va_start (ap, fmt);
   v_display_combinable (src, title, GTK_MESSAGE_WARNING, fmt, ap);
+  va_end (ap);
 }
 
 void display_info_combinable (gxine_msg_source src, const gchar *title,
@@ -538,6 +552,7 @@ void display_info_combinable (gxine_msg_source src, const gchar *title,
   va_list ap;
   va_start (ap, fmt);
   v_display_combinable (src, title, GTK_MESSAGE_INFO, fmt, ap);
+  va_end (ap);
 }
 
 void int_to_timestring (int int_time, char *string_time, gint length)
@@ -607,11 +622,10 @@ gchar *modal_file_dialog (const char *title, gboolean loading, gboolean local,
 			  GtkWidget *parent)
 {
   gchar *ret = NULL;
-  GtkWidget *dbox = gtk_file_chooser_dialog_new_with_backend
+  GtkWidget *dbox = gtk_file_chooser_dialog_new
 	(title, NULL,
 	 loading ? GTK_FILE_CHOOSER_ACTION_OPEN
 		 : GTK_FILE_CHOOSER_ACTION_SAVE,
-	 "gnome-vfs",
 	 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 	 loading ? GTK_STOCK_OPEN : GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 	 NULL);
@@ -641,8 +655,8 @@ GSList *modal_multi_file_dialog (const char *title, gboolean local,
 				 const gchar *pattern, GtkWidget *parent)
 {
   GSList *ret = NULL;
-  GtkWidget *dbox = gtk_file_chooser_dialog_new_with_backend
-	(title, NULL, GTK_FILE_CHOOSER_ACTION_OPEN, "gnome-vfs",
+  GtkWidget *dbox = gtk_file_chooser_dialog_new
+        (title, NULL, GTK_FILE_CHOOSER_ACTION_OPEN,
 	 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 	 GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 	 NULL);
@@ -678,12 +692,12 @@ void get_mrl_from_filesystem (GtkWindow *dbox, GtkEntry *entry)
       suffix = (*suffix && *suffix != '/') ? strchr (suffix, '#') : NULL;
     }
 
-    char *start = g_strconcat (file, suffix, NULL);
+    gchar *start = g_strconcat (file, suffix, NULL);
     gtk_entry_set_text (entry, start);
     gtk_editable_set_position (GTK_EDITABLE (entry), 0);
 
-    free (start);
-    free (file);
+    g_free (start);
+    g_free (file);
   }
 }
 
@@ -725,10 +739,12 @@ void add_table_row_items (GtkWidget *table, gint row, gboolean expand,
   GtkWidget *widget = va_arg (ap, GtkWidget *);
   while (widget)
   {
+    guint ncols = 0;
     GtkWidget *next = va_arg (ap, GtkWidget *);
+    gtk_table_get_size (GTK_TABLE (table), NULL, &ncols);
     gtk_table_attach (GTK_TABLE (table), widget, column,
-		      next ? column + 1 : GTK_TABLE (table)->ncols,
-		      row, row + 1, xattach, yattach, 2, 2);
+                      next ? column + 1 : ncols,
+                      row, row + 1, xattach, yattach, 2, 2);
     yattach = xattach = GTK_FILL;
     ++column;
     widget = next;
@@ -736,32 +752,18 @@ void add_table_row_items (GtkWidget *table, gint row, gboolean expand,
   va_end (ap);
 }
 
-char *asreprintf (char **str, const char *fmt, ...)
+gchar *asreprintf (gchar **str, const char *fmt, ...)
 {
-  char *ret;
+  gchar *ret;
   va_list ap;
 
   va_start (ap, fmt);
   ret = g_strdup_vprintf (fmt, ap);
   va_end (ap);
-  free (*str);
+  g_free (*str);
   return *str = ret;
 }
 
-char **get_driver_ids (const char *const *source)
-{
-  guint size = g_strv_length (source);
-  char **ids = calloc (size + 2, sizeof (char *));
-  int i;
-
-  ids[0] = "auto";
-  for (i = 0; i < size; ++i)
-    ids[i + 1] = strdup (source[i]);
-  ids[i + 1]=0;
-
-  return ids;
-}
-
 #ifdef EXP_STUFF
 char *unique_name (char *base)
 {
@@ -781,7 +783,7 @@ const char *get_copyright_notice (void)
   static gchar *copyright = NULL;
   if (!copyright)
     copyright = g_strdup_printf (_("© 2002-%d the xine project team"),
-				 2008);
+				 2017);
   return copyright;
 }
 
@@ -791,8 +793,10 @@ void config_update_default (const char *key, int value)
 {
   /* warning: accesses xine internals */
   cfg_entry_t *entry = xine->config->lookup_entry (xine->config, key);
+  pthread_mutex_lock(&xine->config->config_lock);
   if (entry)
     entry->num_default = value;
+  pthread_mutex_unlock(&xine->config->config_lock);
 }
 
 /* List a subset of video plugins */
