@@ -21,7 +21,7 @@
 #ifndef GXINE_VIS_H
 #define GXINE_VIS_H
 
-#include <gtk/gtkradioaction.h>
+#include <gtk/gtk.h>
 
 #include "gtkvideo.h"
 
